@@ -38,7 +38,7 @@
 
 #define DEVICE_INFO_MAX_LENGTH 2048
 #define NUM_OF_DEVICE_ID 32
-#define NUM_OPTIONS 6
+#define NUM_OPTIONS 8
 
 #define ERRNO_EXIT(filename) do { \
     printf("IO error on file %s: %s\n", filename, strerror(errno)); \
@@ -46,11 +46,15 @@
   } while(0)
 
 char *kernel_source = NULL;
+size_t kernel_source_len = 0;
+
 char *output_file = NULL;
 cl_uint opencl_device = CL_DEVICE_TYPE_DEFAULT;
 unsigned opencl_device_id = 0;
 int list_devices = 0;
 int list_devices_only = 0;
+int input_is_bitcode = 0;
+int input_is_spirv = 0;
 char *build_options = NULL;
 
 /**********************************************************/
@@ -99,9 +103,12 @@ process_kernel_file(int arg, char **argv, int argc)
 
   char *filename = argv[arg];
   char *ext = ".pocl";
-  kernel_source = poclu_read_file(filename);
+  size_t len = 0;
+  kernel_source = poclu_read_binfile (filename, &len);
   if (!kernel_source)
     ERRNO_EXIT(filename);
+  kernel_source[len] = 0;
+  kernel_source_len = len;
   if (output_file == NULL)
     {
       output_file = malloc (strlen (filename) + strlen (ext) + 2);
@@ -121,6 +128,20 @@ process_help(int arg, char **argv, int argc)
   return 0;
 }
 
+static int
+process_bitcode (int arg, char **argv, int argc)
+{
+  input_is_bitcode = 1;
+  return 0;
+}
+
+static int
+process_spirv (int arg, char **argv, int argc)
+{
+  input_is_spirv = 1;
+  return 0;
+}
+
 static int
 process_output(int arg, char **argv, int argc)
 {
@@ -140,6 +161,8 @@ process_opencl_device(int arg, char **argv, int argc)
   char *opencl_string = argv[arg];
   if (!strcmp(opencl_string, "CL_DEVICE_TYPE_CPU"))
     opencl_device = CL_DEVICE_TYPE_CPU;
+  else if (!strcmp(opencl_string, "CL_DEVICE_TYPE_CUSTOM"))
+    opencl_device = CL_DEVICE_TYPE_CUSTOM;
   else if (!strcmp(opencl_string, "CL_DEVICE_TYPE_GPU"))
     opencl_device = CL_DEVICE_TYPE_GPU;
   else if (!strcmp(opencl_string, "CL_DEVICE_TYPE_ACCELERATOR"))
@@ -213,7 +236,15 @@ static poclcc_option options[NUM_OPTIONS] =
   {process_output, "-o",
    "\t-o <file>\n"
    "\t\tWrite output to <file>\n",
-   2}
+   2},
+  {process_bitcode, "-c",
+   "\t-c\n"
+   "\t\tInput is LLVM IR\n",
+   1},
+  {process_spirv, "-s",
+   "\t-s\n"
+   "\t\tInput is SPIR-V\n",
+   1}
 };
 
 /**********************************************************/
@@ -281,6 +312,7 @@ main(int argc, char **argv)
 //OPENCL STUFF
   cl_platform_id cpPlatform;
   cl_device_id device_ids[NUM_OF_DEVICE_ID];
+  cl_device_id selected_dev;
   cl_context context;
   cl_program program;
   cl_int err;
@@ -323,11 +355,29 @@ main(int argc, char **argv)
   if (list_devices_only)
     return 0;
 
-  context = clCreateContext(0, 1, &device_ids[opencl_device_id], NULL, NULL, &err);
+  selected_dev = device_ids[opencl_device_id];
+  context = clCreateContext (0, 1, &selected_dev, NULL, NULL, &err);
   CHECK_OPENCL_ERROR_IN("clCreateContext");
 
-  program = clCreateProgramWithSource(context, 1, (const char **)&kernel_source, NULL, &err);
-  CHECK_OPENCL_ERROR_IN("clCreateProgramWithSource");
+  if (input_is_spirv)
+    {
+      program = clCreateProgramWithIL (context, (const void *)kernel_source,
+                                       kernel_source_len, &err);
+      CHECK_OPENCL_ERROR_IN ("clCreateProgramWithIL");
+    }
+  else if (input_is_bitcode)
+    {
+      program = clCreateProgramWithBinary (
+          context, 1, &selected_dev, &kernel_source_len,
+          (const unsigned char **)&kernel_source, NULL, &err);
+      CHECK_OPENCL_ERROR_IN ("clCreateProgramWithBinary");
+    }
+  else
+    {
+      program = clCreateProgramWithSource (
+          context, 1, (const char **)&kernel_source, NULL, &err);
+      CHECK_OPENCL_ERROR_IN ("clCreateProgramWithSource");
+    }
 
   err = clBuildProgram (program, 0, NULL, build_options, NULL, NULL);
   if (err != CL_SUCCESS)
