@@ -61,7 +61,7 @@ main (int argc, char **argv)
   srand ((unsigned int)time (NULL));
 
   err = poclu_get_multiple_devices (&platform, &context, 0, &num_devices,
-                                    &devices, &queues);
+                                    &devices, &queues, 0);
 
   CHECK_OPENCL_ERROR_IN ("clCreateContext");
   cl_device_id first_dev = devices[0];
@@ -140,11 +140,13 @@ FINISH:
     {
       CHECK_CL_ERROR (clReleaseCommandQueue (queues[i]));
     }
+  free (queues);
   CHECK_CL_ERROR (clReleaseContext (context));
   CHECK_CL_ERROR (clUnloadPlatformCompiler (platform));
   free (srcA);
   free (srcB);
   free (dst);
+  free (devices);
 
   return err;
 }
