@@ -47,7 +47,11 @@
 /* detects restrict, variadic macros etc */
 #include "pocl_compiler_features.h"
 
-#define POCL_FILENAME_LENGTH 1024
+/* The maximum file, directory and path name lengths. TODO: These should be
+   detected from the filesystem properties of the execution platform. */
+#define POCL_MAX_DIRNAME_LENGTH 255
+#define POCL_MAX_FILENAME_LENGTH (POCL_MAX_DIRNAME_LENGTH)
+#define POCL_MAX_PATHNAME_LENGTH 4096
 
 /* official Khronos ID */
 #ifndef CL_KHRONOS_VENDOR_ID_POCL
@@ -87,6 +91,14 @@ typedef struct pocl_mem_identifier
      it's not */
   void *mem_ptr;
 
+  /* If mem_ptr represents an address in the device global memory which
+     is pinned for the lifetime of the buffer. */
+  int is_pinned;
+
+  /* The device-side memory address (if known). If is_pinned is true, this
+     must be a valid value (note: 0 can be a valid address!). */
+  void *device_addr;
+
   /* Content version tracking. Every write use (clEnqWriteBuffer,
    * clMapBuffer(CL_MAP_WRITE), write_only image, read/write buffers as kernel
    * args etc) increases the version; read uses do not. At command enqueue
@@ -133,7 +145,8 @@ struct _build_program_callback
     void *user_data; /* user supplied data passed to callback function */
 };
 
-#define POCL_KERNEL_DIGEST_SIZE 16
+// same as SHA1_DIGEST_SIZE
+#define POCL_KERNEL_DIGEST_SIZE 20
 typedef uint8_t pocl_kernel_hash_t[POCL_KERNEL_DIGEST_SIZE];
 
 // clEnqueueNDRangeKernel
@@ -153,6 +166,12 @@ typedef struct
   int force_large_grid_wg_func;
 } _cl_command_run;
 
+// clEnqueueCommandBufferKHR
+typedef struct
+{
+  cl_command_buffer_khr buffer;
+} _cl_command_replay;
+
 // clEnqueueNativeKernel
 typedef struct
 {
@@ -167,8 +186,10 @@ typedef struct
 {
   void *__restrict__ dst_host_ptr;
   pocl_mem_identifier *src_mem_id;
+  pocl_mem_identifier *src_content_size_mem_id;
   size_t offset;
   size_t size;
+  size_t *content_size;
 } _cl_command_read;
 
 // clEnqueueWriteBuffer
@@ -338,6 +359,10 @@ typedef struct
   pocl_mem_identifier *src_id;
   pocl_mem_identifier *dst_id;
   pocl_mem_identifier *mem_id;
+  /** For migrating a buffer that has a size buffer as per
+   * cl_pocl_content_size */
+  uint64_t migration_size;
+  pocl_mem_identifier *src_content_size_mem_id;
 } _cl_command_migrate;
 
 typedef struct
@@ -369,6 +394,7 @@ typedef struct
 typedef struct
 {
   void* svm_ptr;
+  size_t size;
 } _cl_command_svm_unmap;
 
 typedef struct
@@ -378,6 +404,19 @@ typedef struct
   size_t size;
 } _cl_command_svm_cpy;
 
+typedef struct
+{
+  const void *__restrict__ src;
+  void *__restrict__ dst;
+  size_t region[3];
+  size_t src_origin[3];
+  size_t dst_origin[3];
+  size_t src_row_pitch;
+  size_t src_slice_pitch;
+  size_t dst_row_pitch;
+  size_t dst_slice_pitch;
+} _cl_command_svm_cpy_rect;
+
 typedef struct
 {
   void *__restrict__ svm_ptr;
@@ -386,10 +425,29 @@ typedef struct
   size_t pattern_size;
 } _cl_command_svm_fill;
 
+typedef struct
+{
+  void *__restrict__ svm_ptr;
+  void *__restrict__ pattern;
+  size_t region[3];
+  size_t origin[3];
+  size_t row_pitch;
+  size_t slice_pitch;
+  size_t pattern_size;
+} _cl_command_svm_fill_rect;
+
+typedef struct
+{
+  const void *ptr;
+  size_t size;
+  cl_mem_advice_intel advice;
+} _cl_command_svm_memadvise;
+
 typedef union
 {
   _cl_command_run run;
   _cl_command_native native;
+  _cl_command_replay replay;
 
   _cl_command_read read;
   _cl_command_write write;
@@ -415,11 +473,15 @@ typedef union
   _cl_command_svm_map svm_map;
   _cl_command_svm_unmap svm_unmap;
   _cl_command_svm_cpy svm_memcpy;
+  _cl_command_svm_cpy_rect svm_memcpy_rect;
   _cl_command_svm_fill svm_fill;
+  _cl_command_svm_fill_rect svm_fill_rect;
   _cl_command_svm_migrate svm_migrate;
+
+  _cl_command_svm_memadvise mem_advise;
 } _cl_command_t;
 
-// one item in the command queue
+// one item in the command queue or command buffer
 typedef struct _cl_command_node _cl_command_node;
 struct _cl_command_node
 {
@@ -427,14 +489,56 @@ struct _cl_command_node
   cl_command_type type;
   _cl_command_node *next; // for linked-list storage
   _cl_command_node *prev;
-  cl_event event;
-  const cl_event *event_wait_list;
+  cl_int buffered;
+
+  /***
+   * Command buffers use sync points as a template for synchronizing commands
+   * within the buffer. Commands outside the buffer can't depend on sync points
+   * and individual commands in the buffer can't depend on events. Because this
+   * struct is used both for recorded and immediately enqueued commands, the
+   * two synchronization mechanisms are made mutually exclusive here.
+   * */
+  union
+  {
+    struct
+    {
+      cl_event event;
+    } event;
+    struct
+    {
+      cl_sync_point_khr sync_point;
+      cl_uint num_sync_points_in_wait_list;
+      cl_sync_point_khr *sync_point_wait_list;
+    } syncpoint;
+  } sync;
   cl_device_id device;
   /* The index of the targeted device in the **program** device list. */
   unsigned program_device_i;
   cl_int ready;
+
+  /* fields needed by buffered commands only */
+
+  /* Which of the command queues in the command buffer's queue list
+   * this command was recorded for. */
+  cl_uint queue_idx;
+  /* List of buffers this command accesses, used for inserting migrations */
+  cl_uint memobj_count;
+  cl_mem *memobj_list;
+  char *readonly_flag_list;
 };
 
+/**
+ * Enumeration for different modes of converting automatic locals
+ */
+typedef enum
+{
+  POCL_AUTOLOCALS_TO_ARGS_NEVER = 0,
+  POCL_AUTOLOCALS_TO_ARGS_ALWAYS = 1,
+  /* convert autolocals to args only if there are dynamic local memory
+   * function arguments in the kernel. */
+  POCL_AUTOLOCALS_TO_ARGS_ONLY_IF_DYNAMIC_LOCALS_PRESENT = 2,
+} pocl_autolocals_to_args_strategy;
+
 #define CLANG_MAJOR LLVM_MAJOR
 #include "_libclang_versions_checks.h"
 
