@@ -109,15 +109,15 @@ typedef uint16_t ushort;
 typedef uint32_t uint;
 
 #ifdef cl_khr_int64
+#if __APPLE__
+typedef unsigned long ulong;
+#else
 typedef uint64_t ulong;
+#endif
 #else
 typedef uint32_t ulong;
 #endif
 
-#ifndef cl_khr_fp16
-typedef short half;
-#endif
-
 #endif
 
 /* The definitions below intentionally lead to errors if these types
