@@ -95,6 +95,10 @@ create_program_skeleton (cl_context context, cl_uint num_devices,
         }
       POCL_GOTO_ERROR_ON((!found), CL_INVALID_DEVICE,
         "device not found in the device list of the context\n");
+      POCL_GOTO_ERROR_ON (
+          (*(device_list[i]->available) == CL_FALSE), CL_DEVICE_NOT_AVAILABLE,
+          "Requested building for device '%s' but it is unavailable.\n",
+          device_list[i]->long_name);
     }
 
   if ((program = (cl_program) calloc (1, sizeof (struct _cl_program))) == NULL)
@@ -120,6 +124,15 @@ create_program_skeleton (cl_context context, cl_uint num_devices,
              == NULL
       || ((program->data = (void **)calloc (num_devices, sizeof (void *)))
           == NULL)
+      || ((program->global_var_total_size
+           = (size_t *)calloc (num_devices, sizeof (size_t)))
+          == NULL)
+      || ((program->llvm_irs
+           = (void *)calloc (num_devices, sizeof (void *)))
+          == NULL)
+      || ((program->gvar_storage
+           = (void *)calloc (num_devices, sizeof (void *)))
+          == NULL)
       || ((program->build_hash
            = (SHA1_digest_t *)calloc (num_devices, sizeof (SHA1_digest_t)))
           == NULL))
@@ -138,7 +151,7 @@ create_program_skeleton (cl_context context, cl_uint num_devices,
 
   TP_CREATE_PROGRAM (context->id, program->id);
 
-  char program_bc_path[POCL_FILENAME_LENGTH];
+  char program_bc_path[POCL_MAX_PATHNAME_LENGTH];
 
   if (allow_empty_binaries && (lengths == NULL) && (binaries == NULL))
     goto SUCCESS;
@@ -216,10 +229,16 @@ ERROR_CLEAN_PROGRAM_AND_BINARIES:
   if (program->pocl_binaries)
     for (i = 0; i < num_devices; ++i)
       POCL_MEM_FREE(program->pocl_binaries[i]);
-  POCL_MEM_FREE(program->pocl_binaries);
+  POCL_MEM_FREE (program->pocl_binaries);
+  POCL_MEM_FREE (program->pocl_binary_sizes);
   POCL_MEM_FREE (program->data);
-  POCL_MEM_FREE(program->pocl_binary_sizes);
-  POCL_MEM_FREE(program);
+  POCL_MEM_FREE (program->global_var_total_size);
+  POCL_MEM_FREE (program->llvm_irs);
+  POCL_MEM_FREE (program->gvar_storage);
+
+  POCL_MEM_FREE (program->build_log);
+  POCL_MEM_FREE (program->build_hash);
+  POCL_MEM_FREE (program);
 ERROR:
   POCL_MEM_FREE(unique_devlist);
   if (errcode_ret != NULL)
