@@ -20,10 +20,14 @@ CL_API_SUFFIX__VERSION_1_0
   POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (command_queue)),
                           CL_INVALID_COMMAND_QUEUE);
 
+  POCL_RETURN_ERROR_COND ((*(command_queue->device->available) == CL_FALSE),
+                          CL_DEVICE_NOT_AVAILABLE);
+
   errcode = pocl_copy_buffer_to_image_common (
       NULL, command_queue, src_buffer, dst_image, src_offset, dst_origin,
       region, num_events_in_wait_list, event_wait_list, event, NULL, NULL,
       NULL, &cmd);
+
   if (errcode != CL_SUCCESS)
     return errcode;
 
