@@ -19,6 +19,9 @@ POname(clEnqueueCopyImage)(cl_command_queue      command_queue ,
   POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (command_queue)),
                           CL_INVALID_COMMAND_QUEUE);
 
+  POCL_RETURN_ERROR_COND ((*(command_queue->device->available) == CL_FALSE),
+                          CL_DEVICE_NOT_AVAILABLE);
+
   errcode = pocl_copy_image_common (NULL, command_queue, src_image, dst_image,
                                     src_origin, dst_origin, region,
                                     num_events_in_wait_list, event_wait_list,
