@@ -50,6 +50,9 @@ POname(clEnqueueMapBuffer)(cl_command_queue command_queue,
   POCL_GOTO_ERROR_COND ((!IS_CL_OBJECT_VALID (command_queue)),
                         CL_INVALID_COMMAND_QUEUE);
 
+  POCL_GOTO_ERROR_COND ((*(command_queue->device->available) == CL_FALSE),
+                        CL_DEVICE_NOT_AVAILABLE);
+
   POCL_GOTO_ERROR_COND ((!IS_CL_OBJECT_VALID (buffer)), CL_INVALID_MEM_OBJECT);
 
   POCL_GOTO_ON_SUB_MISALIGN (buffer, command_queue);
