@@ -24,15 +24,14 @@
 #include "pocl_cl.h"
 #include "pocl_util.h"
 
-CL_API_ENTRY cl_int CL_API_CALL
-POname (clEnqueueSVMMigrateMem) (cl_command_queue command_queue,
-                                 cl_uint num_svm_pointers,
-                                 const void **svm_pointers,
-                                 const size_t *sizes,
-                                 cl_mem_migration_flags flags,
-                                 cl_uint num_events_in_wait_list,
-                                 const cl_event *event_wait_list,
-                                 cl_event *event) CL_API_SUFFIX__VERSION_2_1
+cl_int
+pocl_svm_migrate_mem_common (cl_command_type command_type,
+                             cl_command_queue command_queue,
+                             cl_uint num_svm_pointers,
+                             const void **svm_pointers, const size_t *sizes,
+                             cl_mem_migration_flags flags,
+                             cl_uint num_events_in_wait_list,
+                             const cl_event *event_wait_list, cl_event *event)
 {
   unsigned i;
   cl_int errcode;
@@ -40,8 +39,10 @@ POname (clEnqueueSVMMigrateMem) (cl_command_queue command_queue,
   POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (command_queue)),
                           CL_INVALID_COMMAND_QUEUE);
 
+  cl_context context = command_queue->context;
+
   POCL_RETURN_ERROR_ON (
-      (command_queue->context->svm_allocdev == NULL), CL_INVALID_OPERATION,
+      (context->svm_allocdev == NULL), CL_INVALID_OPERATION,
       "None of the devices in this context is SVM-capable\n");
 
   POCL_RETURN_ERROR_COND ((svm_pointers == NULL), CL_INVALID_VALUE);
@@ -56,6 +57,10 @@ POname (clEnqueueSVMMigrateMem) (cl_command_queue command_queue,
   for (i = 0; i < num_svm_pointers; ++i)
     {
       POCL_RETURN_ERROR_COND ((svm_pointers[i] == NULL), CL_INVALID_VALUE);
+      size_t size = sizes ? sizes[i] : 1;
+      errcode = pocl_svm_check_pointer (context, svm_pointers[i], size, NULL);
+      if (errcode != CL_SUCCESS)
+        return errcode;
     }
 
   errcode = pocl_check_event_wait_list (command_queue, num_events_in_wait_list,
@@ -64,9 +69,9 @@ POname (clEnqueueSVMMigrateMem) (cl_command_queue command_queue,
     return errcode;
 
   _cl_command_node *cmd = NULL;
-  errcode = pocl_create_command (
-      &cmd, command_queue, CL_COMMAND_SVM_MIGRATE_MEM, event,
-      num_events_in_wait_list, event_wait_list, 0, NULL, NULL);
+  errcode = pocl_create_command (&cmd, command_queue, command_type, event,
+                                 num_events_in_wait_list, event_wait_list, 0,
+                                 NULL, NULL);
 
   if (errcode != CL_SUCCESS)
     {
@@ -92,4 +97,20 @@ POname (clEnqueueSVMMigrateMem) (cl_command_queue command_queue,
 
   return CL_SUCCESS;
 }
+
+CL_API_ENTRY cl_int CL_API_CALL
+POname (clEnqueueSVMMigrateMem) (cl_command_queue command_queue,
+                                 cl_uint num_svm_pointers,
+                                 const void **svm_pointers,
+                                 const size_t *sizes,
+                                 cl_mem_migration_flags flags,
+                                 cl_uint num_events_in_wait_list,
+                                 const cl_event *event_wait_list,
+                                 cl_event *event) CL_API_SUFFIX__VERSION_2_1
+{
+  return pocl_svm_migrate_mem_common (
+      CL_COMMAND_SVM_MIGRATE_MEM, command_queue, num_svm_pointers,
+      svm_pointers, sizes, flags, num_events_in_wait_list, event_wait_list,
+      event);
+}
 POsym (clEnqueueSVMMigrateMem)
