@@ -1,17 +1,18 @@
 /* OpenCL runtime library: clGetDeviceInfo()
 
    Copyright (c) 2011-2012 Kalle Raiskila and Pekka Jääskeläinen
-   
+                 2022-2023 Pekka Jääskeläinen / Intel Finland Oy
+
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -52,6 +53,9 @@ POname(clGetDeviceInfo)(cl_device_id   device,
 {
   POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (device)), CL_INVALID_DEVICE);
 
+  POCL_RETURN_ERROR_COND ((*(device->available) == CL_FALSE),
+                          CL_DEVICE_NOT_AVAILABLE);
+
   switch (param_name)
   {
   case CL_DEVICE_IMAGE_SUPPORT:
@@ -164,7 +168,7 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_ENDIAN_LITTLE                     :
     POCL_RETURN_GETINFO(cl_uint, device->endian_little);
   case CL_DEVICE_AVAILABLE                         :
-    POCL_RETURN_GETINFO(cl_bool, device->available);
+    POCL_RETURN_GETINFO (cl_bool, *(device->available));
   case CL_DEVICE_COMPILER_AVAILABLE                :
     POCL_RETURN_GETINFO(cl_bool, device->compiler_available);
   case CL_DEVICE_LINKER_AVAILABLE                  :
@@ -172,6 +176,16 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_EXECUTION_CAPABILITIES            :
     POCL_RETURN_GETINFO(cl_device_exec_capabilities, device->execution_capabilities);
 
+  case CL_DEVICE_SINGLE_FP_ATOMIC_CAPABILITIES_EXT:
+    POCL_RETURN_GETINFO(cl_device_fp_atomic_capabilities_ext,
+                        device->single_fp_atomic_caps);
+  case CL_DEVICE_DOUBLE_FP_ATOMIC_CAPABILITIES_EXT:
+    POCL_RETURN_GETINFO(cl_device_fp_atomic_capabilities_ext,
+                        device->double_fp_atomic_caps);
+  case CL_DEVICE_HALF_FP_ATOMIC_CAPABILITIES_EXT:
+    POCL_RETURN_GETINFO(cl_device_fp_atomic_capabilities_ext,
+                        device->half_fp_atomic_caps);
+
   case CL_DEVICE_NAME:
     POCL_RETURN_GETINFO_STR(device->long_name);
    
@@ -290,11 +304,11 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_QUEUE_ON_DEVICE_MAX_SIZE:
     POCL_RETURN_GETINFO(cl_uint, device->dev_queue_max_size);
   case CL_DEVICE_PREFERRED_GLOBAL_ATOMIC_ALIGNMENT:
-    POCL_RETURN_GETINFO(cl_uint, 0);
+    POCL_RETURN_GETINFO(cl_uint, 64);
   case CL_DEVICE_PREFERRED_LOCAL_ATOMIC_ALIGNMENT:
-    POCL_RETURN_GETINFO(cl_uint, 0);
+    POCL_RETURN_GETINFO(cl_uint, 64);
   case CL_DEVICE_PREFERRED_PLATFORM_ATOMIC_ALIGNMENT:
-    POCL_RETURN_GETINFO(cl_uint, 0);
+    POCL_RETURN_GETINFO(cl_uint, 64);
 
   case CL_DEVICE_QUEUE_ON_DEVICE_PROPERTIES:
     POCL_RETURN_GETINFO(cl_command_queue_properties, device->on_dev_queue_props);
@@ -333,7 +347,7 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_WORK_GROUP_COLLECTIVE_FUNCTIONS_SUPPORT:
     POCL_RETURN_GETINFO (cl_bool, CL_FALSE);
   case CL_DEVICE_GENERIC_ADDRESS_SPACE_SUPPORT:
-    POCL_RETURN_GETINFO (cl_bool, CL_FALSE);
+    POCL_RETURN_GETINFO (cl_bool, device->generic_as_support);
   case CL_DEVICE_DEVICE_ENQUEUE_CAPABILITIES:
     POCL_RETURN_GETINFO (cl_uint, 0);
   case CL_DEVICE_PIPE_SUPPORT:
