@@ -28,7 +28,8 @@ POname(clRetainProgram)(cl_program program) CL_API_SUFFIX__VERSION_1_0
 {
   POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (program)), CL_INVALID_PROGRAM);
   POCL_RETAIN_OBJECT(program);
-  POCL_MSG_PRINT_REFCOUNTS ("Retain Program %p  : %d\n", program, program->pocl_refcount);
+  POCL_MSG_PRINT_REFCOUNTS ("Retain Program %" PRId64 " (%p), Refcount: %d\n",
+                            program->id, program, program->pocl_refcount);
   return CL_SUCCESS;
 }
 POsym(clRetainProgram)
