@@ -67,6 +67,8 @@ struct data {
 
   /* printf buffer */
   void *printf_buffer;
+
+  cl_bool available;
 };
 
 typedef struct _pocl_basic_usm_allocation_t
@@ -144,7 +146,7 @@ pocl_basic_init_device_ops(struct pocl_device_ops *ops)
   ops->svm_unmap = NULL;
   ops->svm_advise = NULL;
   ops->svm_migrate = NULL;
-  ops->svm_copy = pocl_basic_svm_copy;
+  ops->svm_copy = pocl_driver_svm_copy;
   ops->svm_fill = pocl_driver_svm_fill;
 
   ops->create_kernel = NULL;
@@ -203,7 +205,9 @@ pocl_basic_init (unsigned j, cl_device_id device, const char* parameters)
     return CL_OUT_OF_HOST_MEMORY;
 
   d->current_kernel = NULL;
+  d->available = CL_TRUE;
   device->data = d;
+  device->available = &d->available;
 
   pocl_init_default_device_infos (device);
 
@@ -227,7 +231,7 @@ pocl_basic_init (unsigned j, cl_device_id device, const char* parameters)
 
 #if (HOST_DEVICE_CL_VERSION_MAJOR >= 3)
   device->features = HOST_DEVICE_FEATURES_30;
-  device->program_scope_variables_pass = CL_TRUE;
+  device->run_program_scope_variables_pass = CL_TRUE;
   device->generic_as_support = CL_TRUE;
 
   pocl_setup_opencl_c_with_version (device, CL_TRUE);
@@ -268,6 +272,17 @@ pocl_basic_init (unsigned j, cl_device_id device, const char* parameters)
   device->svm_caps = CL_DEVICE_SVM_COARSE_GRAIN_BUFFER
                      | CL_DEVICE_SVM_FINE_GRAIN_BUFFER
                      | CL_DEVICE_SVM_ATOMICS;
+
+  if (strstr (HOST_DEVICE_EXTENSIONS, "cl_ext_float_atomics")
+      != NULL) {
+    device->single_fp_atomic_caps = device->double_fp_atomic_caps =
+      CL_DEVICE_GLOBAL_FP_ATOMIC_LOAD_STORE_EXT |
+      CL_DEVICE_GLOBAL_FP_ATOMIC_ADD_EXT |
+      CL_DEVICE_GLOBAL_FP_ATOMIC_MIN_MAX_EXT |
+      CL_DEVICE_LOCAL_FP_ATOMIC_LOAD_STORE_EXT |
+      CL_DEVICE_LOCAL_FP_ATOMIC_ADD_EXT |
+      CL_DEVICE_LOCAL_FP_ATOMIC_MIN_MAX_EXT;
+  }
 #endif
 
   if (strstr (HOST_DEVICE_EXTENSIONS, "cl_intel_unified_shared_memory")
@@ -548,7 +563,7 @@ pocl_basic_uninit (unsigned j, cl_device_id device)
 }
 
 cl_int
-pocl_basic_reinit (unsigned j, cl_device_id device)
+pocl_basic_reinit (unsigned j, cl_device_id device, const char *parameters)
 {
   struct data *d = (struct data *)calloc (1, sizeof (struct data));
   if (d == NULL)
@@ -648,7 +663,9 @@ pocl_basic_notify (cl_device_id device, cl_event event, cl_event finished)
           POCL_LOCK (d->cq_lock);
           CDL_DELETE (d->command_list, node);
           CDL_PREPEND (d->ready_list, node);
+          POCL_UNLOCK_OBJ (event);
           basic_command_scheduler (d);
+          POCL_LOCK_OBJ (event);
           POCL_UNLOCK (d->cq_lock);
         }
       return;
@@ -975,13 +992,6 @@ pocl_basic_usm_free (cl_device_id dev, void *svm_ptr)
   pocl_aligned_free (svm_ptr);
 }
 
-void
-pocl_basic_svm_copy (cl_device_id dev, void *__restrict__ dst,
-                     const void *__restrict__ src, size_t size)
-{
-  memcpy (dst, src, size);
-}
-
 static cl_bitfield
 pocl_basic_get_mem_caps (cl_device_id device, cl_device_info Type)
 {
