@@ -387,8 +387,10 @@ pocl_driver_free_mapping_ptr (void *data, pocl_mem_identifier *mem_id,
   if (map->host_ptr == NULL)
     return CL_SUCCESS;
 
-  if ((mem->mem_host_ptr != NULL)
-      && map->host_ptr != (mem->mem_host_ptr + map->offset))
+  /* e.g. remote never has a mem_host_ptr but can have a map host_ptr */
+  if (((mem->mem_host_ptr != NULL)
+       && map->host_ptr != (mem->mem_host_ptr + map->offset))
+      || (mem->mem_host_ptr == NULL && map->host_ptr != NULL))
     pocl_aligned_free (map->host_ptr);
 
   map->host_ptr = NULL;
@@ -416,6 +418,12 @@ pocl_driver_alloc_mem_obj (cl_device_id device, cl_mem mem, void *host_ptr)
 
   p->version = mem->mem_host_ptr_version;
   p->mem_ptr = mem->mem_host_ptr;
+  p->device_addr = p->mem_ptr;
+
+  /* If requesting device pinned memory, mark the mem such so it won't
+     get migrated away before being freed. */
+  if (mem->is_device_pinned)
+    p->is_pinned = 1;
 
   POCL_MSG_PRINT_MEMORY ("Basic device ALLOC %p / size %zu \n", p->mem_ptr,
                          mem->size);
@@ -446,6 +454,15 @@ pocl_driver_svm_fill (cl_device_id dev, void *__restrict__ svm_ptr,
   pocl_driver_memfill (dev->data, &temp, NULL, size, 0, pattern, pattern_size);
 }
 
+void
+pocl_driver_svm_copy (cl_device_id dev,
+                      void *__restrict__ dst,
+                      const void *__restrict__ src,
+                      size_t size)
+{
+  memcpy (dst, src, size);
+}
+
 /* These are implementations of compilation callbacks for all devices
  * that support compilation via LLVM. They take care of compilation/linking
  * of source/binary/spir down to parallel.bc level.
@@ -486,7 +503,7 @@ pocl_regen_spirv_binary (cl_program program, cl_uint device_i)
 {
 #ifdef LLVM_SPIRV
   int errcode = CL_SUCCESS;
-
+  cl_device_id device = program->devices[device_i];
   int spec_constants_changed = 0;
   char concated_spec_const_option[MAX_SPEC_CONST_CMDLINE_LEN];
   concated_spec_const_option[0] = 0;
@@ -494,12 +511,15 @@ pocl_regen_spirv_binary (cl_program program, cl_uint device_i)
   char unlinked_program_bc_temp[POCL_MAX_PATHNAME_LENGTH];
   program_bc_spirv[0] = 0;
   unlinked_program_bc_temp[0] = 0;
-  /* using --spirv-target-env=CL2.0 here would enable proper OpenCL 2.0
-   * atomics, unfortunately it also enables generic ptrs which PoCL doesn't
-   * support */
+
+  /* using --spirv-target-env=CL2.0 here enables llvm-spirv to produce proper
+   * OpenCL 2.0 atomics, unfortunately it also enables generic ptrs, which not
+   * all PoCL devices support, hence check the device */
+  char* spirv_target_env = (device->generic_as_support != CL_FALSE) ?
+                        "--spirv-target-env=CL2.0" :  "--spirv-target-env=CL1.2";
   char *args[8] = { LLVM_SPIRV,
                     concated_spec_const_option,
-                    "--spirv-target-env=CL1.2",
+                    spirv_target_env,
                     "-r", "-o",
                     unlinked_program_bc_temp,
                     program_bc_spirv,
@@ -579,7 +599,7 @@ pocl_llvm_convert_and_link_ir (cl_program program, cl_uint device_i,
   if (program->binaries[device_i])
     {
       int spir_binary
-          = bitcode_is_triple ((char *)program->binaries[device_i],
+          = pocl_bitcode_is_triple ((char *)program->binaries[device_i],
                                program->binary_sizes[device_i], "spir");
       if (spir_binary)
         {
@@ -854,7 +874,7 @@ pocl_driver_supports_binary (cl_device_id device, size_t length,
 
 #ifdef ENABLE_SPIR
   /* SPIR binary is supported if the device has cl_khr_spir */
-  if (bitcode_is_triple (binary, length, "spir"))
+  if (pocl_bitcode_is_triple (binary, length, "spir"))
     {
       if (strstr (device->extensions, "cl_khr_spir") == NULL)
         {
@@ -871,7 +891,7 @@ pocl_driver_supports_binary (cl_device_id device, size_t length,
 
   /* LLVM IR can be supported by the driver, if the triple matches */
   if (device->llvm_target_triplet
-      && bitcode_is_triple (binary, length, device->llvm_target_triplet))
+      && pocl_bitcode_is_triple (binary, length, device->llvm_target_triplet))
     return 1;
 
   return 0;
@@ -1117,7 +1137,7 @@ pocl_driver_build_gvar_init_kernel (cl_program program, cl_uint dev_i,
                                     cl_device_id device, gvar_init_callback_t callback)
 {
 #ifdef ENABLE_HOST_CPU_DEVICES
-  if (device->program_scope_variables_pass == CL_FALSE)
+  if (device->run_program_scope_variables_pass == CL_FALSE)
     return;
 
   if (program->global_var_total_size[dev_i] > 0 && program->gvar_storage[dev_i] == NULL)
