@@ -1,40 +1,42 @@
 /* pocl-ptx-gen.cc - PTX code generation functions
 
-   Copyright (c) 2016-2017 James Price / University of Bristol
+    Copyright (c) 2016-2017 James Price / University of Bristol
 
-   Permission is hereby granted, free of charge, to any person obtaining a copy
-   of this software and associated documentation files (the "Software"), to deal
-   in the Software without restriction, including without limitation the rights
-   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-   copies of the Software, and to permit persons to whom the Software is
-   furnished to do so, subject to the following conditions:
+    Permission is hereby granted, free of charge, to any person obtaining a copy
+    of this software and associated documentation files (the "Software"), to
+    deal in the Software without restriction, including without limitation the
+    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+    sell copies of the Software, and to permit persons to whom the Software is
+    furnished to do so, subject to the following conditions:
 
-   The above copyright notice and this permission notice shall be included in
-   all copies or substantial portions of the Software.
+    The above copyright notice and this permission notice shall be included in
+    all copies or substantial portions of the Software.
 
-   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-   THE SOFTWARE.
+    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+    IN THE SOFTWARE.
 */
 
 #include "config.h"
 
-#include "LLVMUtils.h"
 #include "common.h"
 #include "pocl-ptx-gen.h"
 #include "pocl.h"
+#include "pocl_debug.h"
 #include "pocl_file_util.h"
+#include "pocl_llvm_api.h"
 #include "pocl_runtime_config.h"
 
 #include "llvm/Bitcode/BitcodeReader.h"
 #include "llvm/IR/Constants.h"
-#include "llvm/IR/InstIterator.h"
+#include "llvm/IR/Function.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/LegacyPassManager.h"
+#include "llvm/IR/Metadata.h"
 #include "llvm/IR/Module.h"
 #include "llvm/IR/Verifier.h"
 #include "llvm/Linker/Linker.h"
@@ -43,78 +45,119 @@
 #else
 #include "llvm/Support/TargetRegistry.h"
 #endif
-#include "llvm/Support/FileSystem.h"
-#include "llvm/Support/MemoryBuffer.h"
-#include "llvm/Support/TargetSelect.h"
 #ifndef LLVM_OLDER_THAN_11_0
 #include "llvm/Support/Alignment.h"
 #endif
 #include "llvm/Target/TargetMachine.h"
 #include "llvm/Target/TargetOptions.h"
 #include "llvm/Transforms/IPO.h"
-#include "llvm/Transforms/IPO/PassManagerBuilder.h"
 #include "llvm/Transforms/Utils/Cloning.h"
 
+#include <llvm/Analysis/TargetLibraryInfo.h>
+#include <llvm/Analysis/TargetTransformInfo.h>
+
+#if LLVM_MAJOR >= 17
+#include <llvm/Transforms/IPO/Internalize.h>
+#endif
+
+#include "LLVMUtils.h"
+
 #include <set>
+#include <optional>
 
 namespace llvm {
 extern ModulePass *createNVVMReflectPass(const StringMap<int> &Mapping);
 }
 
-static void addKernelAnnotations(llvm::Module *Module, const char *KernelName);
-static void fixConstantMemArgs(llvm::Module *Module, const char *KernelName);
-static void fixLocalMemArgs(llvm::Module *Module, const char *KernelName);
+typedef std::map<std::string, std::vector<size_t>> AlignmentMapT;
+
+static void addKernelAnnotations(llvm::Module *Module);
+static void fixConstantMemArgs(llvm::Module *Module);
+static void fixLocalMemArgs(llvm::Module *Module);
 static void fixPrintF(llvm::Module *Module);
-static void handleGetWorkDim(llvm::Module *Module, const char *KernelName);
-static void linkLibDevice(llvm::Module *Module, const char *KernelName,
-                          const char *LibDevicePath);
+static void handleGetWorkDim(llvm::Module *Module);
+static int linkLibDevice(llvm::Module *Module, const char *LibDevicePath);
 static void mapLibDeviceCalls(llvm::Module *Module);
+static void createAlignmentMap(llvm::Module *Module,
+                               AlignmentMapT *AlignmentMap);
 
-int pocl_ptx_gen(const char *BitcodeFilename, const char *PTXFilename,
-                 const char *KernelName, const char *Arch,
-                 const char *LibDevicePath, int HasOffsets) {
-  llvm::ErrorOr<std::unique_ptr<llvm::MemoryBuffer>> Buffer =
-      llvm::MemoryBuffer::getFile(BitcodeFilename);
-  if (!Buffer) {
-    POCL_MSG_ERR("[CUDA] ptx-gen: failed to open bitcode file\n");
-    return 1;
+namespace pocl {
+extern bool isGVarUsedByFunction(llvm::GlobalVariable *GVar, llvm::Function *F);
+extern llvm::ModulePass *
+createAutomaticLocalsPass(pocl_autolocals_to_args_strategy autolocals_to_args);
+extern bool isKernelToProcess(const llvm::Function &F);
+} // namespace pocl
+
+static bool verifyModule(llvm::Module *Module, const char *step) {
+  std::string Error;
+  llvm::raw_string_ostream Errs(Error);
+  if (llvm::verifyModule(*Module, &Errs)) {
+    POCL_MSG_ERR("[CUDA] ptx-gen step %s: module verification FAILED\n%s\n",
+                 step, Error.c_str());
+    return false;
+  } else {
+    // POCL_MSG_PRINT_CUDA("[CUDA] ptx-gen: STEP %s: module verification
+    // PASSED\n", step);
+    return true;
   }
+}
 
-  // Load the LLVM bitcode module.
-  llvm::LLVMContext Context;
-  llvm::Expected<std::unique_ptr<llvm::Module>> Module =
-      parseBitcodeFile(Buffer->get()->getMemBufferRef(), Context);
+int pocl_ptx_gen(void *llvm_module, const char *PTXFilename, const char *Arch,
+                 const char *LibDevicePath, int HasOffsets,
+                 void **AlignmentMapPtr) {
+
+  llvm::Module *Module = (llvm::Module *)llvm_module;
   if (!Module) {
     POCL_MSG_ERR("[CUDA] ptx-gen: failed to load bitcode\n");
-    return 1;
+    return CL_BUILD_PROGRAM_FAILURE;
   }
+  bool VerifyMod =
+      pocl_get_bool_option("POCL_LLVM_VERIFY", LLVM_VERIFY_MODULE_DEFAULT);
+
+  AlignmentMapT *A = new AlignmentMapT;
+  assert(*AlignmentMapPtr == nullptr);
+  *AlignmentMapPtr = A;
+  createAlignmentMap(Module, A);
+  if (VerifyMod && !verifyModule(Module, "getAlignmentMap"))
+    return CL_BUILD_PROGRAM_FAILURE;
 
   // Apply transforms to prepare for lowering to PTX.
-  fixPrintF(Module->get());
-  fixConstantMemArgs(Module->get(), KernelName);
-  fixLocalMemArgs(Module->get(), KernelName);
-  handleGetWorkDim(Module->get(), KernelName);
-  addKernelAnnotations(Module->get(), KernelName);
-  mapLibDeviceCalls(Module->get());
-  linkLibDevice(Module->get(), KernelName, LibDevicePath);
+  fixPrintF(Module);
+  if (VerifyMod && !verifyModule(Module, "fixPrintF"))
+    return CL_BUILD_PROGRAM_FAILURE;
+
+  fixConstantMemArgs(Module);
+  if (VerifyMod && !verifyModule(Module, "fixConstantMemArgs"))
+    return CL_BUILD_PROGRAM_FAILURE;
+
+  fixLocalMemArgs(Module);
+  if (VerifyMod && !verifyModule(Module, "fixLocalMemArgs"))
+    return CL_BUILD_PROGRAM_FAILURE;
+
+  handleGetWorkDim(Module);
+  if (VerifyMod && !verifyModule(Module, "handleGetWorkDim"))
+    return CL_BUILD_PROGRAM_FAILURE;
+
+  addKernelAnnotations(Module);
+  if (VerifyMod && !verifyModule(Module, "addAnnotations"))
+    return CL_BUILD_PROGRAM_FAILURE;
+
+  mapLibDeviceCalls(Module);
+  if (VerifyMod && !verifyModule(Module, "mapLibDeviceCalls"))
+    return CL_BUILD_PROGRAM_FAILURE;
+
+  if (linkLibDevice(Module, LibDevicePath) != 0)
+    return CL_BUILD_PROGRAM_FAILURE;
+  if (VerifyMod && !verifyModule(Module, "linkLibDevice"))
+    return CL_BUILD_PROGRAM_FAILURE;
+
   if (pocl_get_bool_option("POCL_CUDA_DUMP_NVVM", 0)) {
     std::string ModuleString;
     llvm::raw_string_ostream ModuleStringStream(ModuleString);
-    (*Module)->print(ModuleStringStream, NULL);
+    Module->print(ModuleStringStream, NULL);
     POCL_MSG_PRINT_INFO("NVVM module:\n%s\n", ModuleString.c_str());
   }
 
-  std::string Error;
-
-  // Verify module.
-  if (pocl_get_bool_option("POCL_CUDA_VERIFY_MODULE", 1)) {
-    llvm::raw_string_ostream Errs(Error);
-    if (llvm::verifyModule(*Module->get(), &Errs)) {
-      POCL_MSG_ERR("\n%s\n", Error.c_str());
-      POCL_ABORT("[CUDA] ptx-gen: module verification failed\n");
-    }
-  }
-
 #ifdef LLVM_OLDER_THAN_11_0
   llvm::StringRef Triple =
       (sizeof(void *) == 8) ? "nvptx64-nvidia-cuda" : "nvptx-nvidia-cuda";
@@ -123,6 +166,7 @@ int pocl_ptx_gen(const char *BitcodeFilename, const char *PTXFilename,
       (sizeof(void *) == 8) ? "nvptx64-nvidia-cuda" : "nvptx-nvidia-cuda";
 #endif
 
+  std::string Error;
   // Get NVPTX target.
   const llvm::Target *Target =
       llvm::TargetRegistry::lookupTarget(Triple, Error);
@@ -130,49 +174,59 @@ int pocl_ptx_gen(const char *BitcodeFilename, const char *PTXFilename,
   if (!Target) {
     POCL_MSG_ERR("[CUDA] ptx-gen: failed to get target\n");
     POCL_MSG_ERR("%s\n", Error.c_str());
-    return 1;
+    return CL_BUILD_PROGRAM_FAILURE;
   }
 
   // TODO: Set options?
   llvm::TargetOptions Options;
 
   // TODO: CPU and features?
+#ifdef LLVM_OLDER_THAN_16_0
   std::unique_ptr<llvm::TargetMachine> Machine(
       Target->createTargetMachine(Triple, Arch, "+ptx40", Options, llvm::None));
-
+#else
+  std::unique_ptr<llvm::TargetMachine> Machine(
+      Target->createTargetMachine(Triple, Arch, "+ptx40", Options, std::nullopt));
+#endif
   llvm::legacy::PassManager Passes;
 
   // Add pass to emit PTX.
   llvm::SmallVector<char, 4096> Data;
   llvm::raw_svector_ostream PTXStream(Data);
   if (Machine->addPassesToEmitFile(Passes, PTXStream,
-#ifndef LLVM_OLDER_THAN_7_0
                                    nullptr,
-#endif
-#ifdef LLVM_OLDER_THAN_10_0
-                                   llvm::TargetMachine::CGFT_AssemblyFile)) {
-#else
                                    llvm::CGFT_AssemblyFile)) {
-#endif
     POCL_MSG_ERR("[CUDA] ptx-gen: failed to add passes\n");
-    return 1;
+    return CL_BUILD_PROGRAM_FAILURE;
   }
 
   // Run passes.
-  Passes.run(**Module);
+  Passes.run(*Module);
 
 #ifdef LLVM_OLDER_THAN_11_0
   std::string PTX = PTXStream.str();
-  return pocl_write_file(PTXFilename, PTX.c_str(), PTX.size(), 0, 0);
+  const char *Content = PTX.c_str();
+  size_t ContentSize = PTX.size();
 #else
   llvm::StringRef PTX = PTXStream.str();
-  return pocl_write_file(PTXFilename, PTX.data(), PTX.size(), 0, 0);
+  const char *Content = PTX.data();
+  size_t ContentSize = PTX.size();
 #endif
+
+  if (pocl_write_file(PTXFilename, Content, ContentSize, 0, 0)) {
+    POCL_MSG_ERR("[CUDA] ptx-gen: failed to write final PTX into %s\n",
+                 PTXFilename);
+    return CL_BUILD_PROGRAM_FAILURE;
+  }
+
+  return CL_SUCCESS;
 }
 
 // Add the metadata needed to mark a function as a kernel in PTX.
-void addKernelAnnotations(llvm::Module *Module, const char *KernelName) {
+void addKernelAnnotations(llvm::Module *Module) {
   llvm::LLVMContext &Context = Module->getContext();
+  llvm::Constant *One =
+      llvm::ConstantInt::getSigned(llvm::Type::getInt32Ty(Context), 1);
 
   // Remove existing nvvm.annotations metadata since it is sometimes corrupt.
   auto *Annotations = Module->getNamedMetadata("nvvm.annotations");
@@ -182,20 +236,20 @@ void addKernelAnnotations(llvm::Module *Module, const char *KernelName) {
   // Add nvvm.annotations metadata to mark kernel entry point.
   Annotations = Module->getOrInsertNamedMetadata("nvvm.annotations");
 
-  // Get handle to function.
-  auto *Function = Module->getFunction(KernelName);
-  if (!Function)
-    POCL_ABORT("[CUDA] ptx-gen: kernel function not found in module\n");
+  for (auto &FI : Module->functions()) {
+    if (!pocl::isKernelToProcess(FI))
+      continue;
 
-  // Create metadata.
-  llvm::Constant *One =
-      llvm::ConstantInt::getSigned(llvm::Type::getInt32Ty(Context), 1);
-  llvm::Metadata *FuncMD = llvm::ValueAsMetadata::get(Function);
-  llvm::Metadata *NameMD = llvm::MDString::get(Context, "kernel");
-  llvm::Metadata *OneMD = llvm::ConstantAsMetadata::get(One);
+    llvm::Function *Function = &FI;
 
-  llvm::MDNode *Node = llvm::MDNode::get(Context, {FuncMD, NameMD, OneMD});
-  Annotations->addOperand(Node);
+    // Create metadata.
+    llvm::Metadata *FuncMD = llvm::ValueAsMetadata::get(Function);
+    llvm::Metadata *NameMD = llvm::MDString::get(Context, "kernel");
+    llvm::Metadata *OneMD = llvm::ConstantAsMetadata::get(One);
+
+    llvm::MDNode *Node = llvm::MDNode::get(Context, {FuncMD, NameMD, OneMD});
+    Annotations->addOperand(Node);
+  }
 }
 
 // PTX doesn't support variadic functions, so we need to modify the IR to
@@ -233,8 +287,12 @@ void fixPrintF(llvm::Module *Module) {
   NewPrintF->takeName(OldPrintF);
 
   // Take function body from old function.
+#ifdef LLVM_OLDER_THAN_16_0
   NewPrintF->getBasicBlockList().splice(NewPrintF->begin(),
                                         OldPrintF->getBasicBlockList());
+#else
+  NewPrintF->splice(NewPrintF->begin(), OldPrintF);
+#endif
 
   // Create i32 to hold current argument index.
 #ifdef LLVM_OLDER_THAN_11_0
@@ -330,11 +388,7 @@ void fixPrintF(llvm::Module *Module) {
     if (!Call)
       continue;
 
-#ifndef LLVM_OLDER_THAN_8_0
     unsigned NumArgs = Call->arg_size() - 1;
-#else
-    unsigned NumArgs = Call->getNumArgOperands() - 1;
-#endif
 
     llvm::Value *Format = Call->getArgOperand(0);
 
@@ -462,6 +516,7 @@ void fixPrintF(llvm::Module *Module) {
   VPrintF->eraseFromParent();
 }
 
+// TODO broken, replaces in whole module not just  1 function
 // Replace all load users of a scalar global variable with new value.
 static void replaceScalarGlobalVar(llvm::Module *Module, const char *Name,
                                    llvm::Value *NewValue) {
@@ -481,49 +536,60 @@ static void replaceScalarGlobalVar(llvm::Module *Module, const char *Name,
 }
 
 // Add an extra kernel argument for the dimensionality.
-void handleGetWorkDim(llvm::Module *Module, const char *KernelName) {
-  llvm::Function *Function = Module->getFunction(KernelName);
-  if (!Function)
-    POCL_ABORT("[CUDA] ptx-gen: kernel function not found in module\n");
+void handleGetWorkDim(llvm::Module *Module) {
 
-  // Add additional argument for the work item dimensionality.
-  llvm::FunctionType *FunctionType = Function->getFunctionType();
-  std::vector<llvm::Type *> ArgumentTypes(FunctionType->param_begin(),
-                                          FunctionType->param_end());
-  ArgumentTypes.push_back(llvm::Type::getInt32Ty(Module->getContext()));
-
-  // Create new function.
-  llvm::FunctionType *NewFunctionType =
-      llvm::FunctionType::get(Function->getReturnType(), ArgumentTypes, false);
-  llvm::Function *NewFunction = llvm::Function::Create(
-      NewFunctionType, Function->getLinkage(), Function->getName(), Module);
-  NewFunction->takeName(Function);
-
-  // Map function arguments.
-  llvm::ValueToValueMapTy VV;
-  llvm::Function::arg_iterator OldArg;
-  llvm::Function::arg_iterator NewArg;
-  for (OldArg = Function->arg_begin(), NewArg = NewFunction->arg_begin();
-       OldArg != Function->arg_end(); NewArg++, OldArg++) {
-    NewArg->takeName(&*OldArg);
-    VV[&*OldArg] = &*NewArg;
-  }
-
-  // Clone function.
-  llvm::SmallVector<llvm::ReturnInst *, 1> RI;
-  CloneFunctionIntoAbs(NewFunction, Function, VV, RI);
-
-  Function->eraseFromParent();
+  llvm::SmallVector<llvm::Function *, 8> FunctionsToErase;
 
   auto WorkDimVar = Module->getGlobalVariable("_work_dim");
-  if (!WorkDimVar)
+  if (WorkDimVar == nullptr)
     return;
 
-  // Replace uses of the global offset variables with the new arguments.
-  NewArg->setName("work_dim");
-  replaceScalarGlobalVar(Module, "_work_dim", (&*NewArg++));
+  for (auto &FI : Module->functions()) {
+    if (!pocl::isKernelToProcess(FI))
+      continue;
 
-  // TODO: What if get_work_dim() is called from a non-kernel function?
+    llvm::Function *Function = &FI;
+    if (!pocl::isGVarUsedByFunction(WorkDimVar, Function))
+      continue;
+
+    // Add additional argument for the work item dimensionality.
+    llvm::FunctionType *FunctionType = Function->getFunctionType();
+    std::vector<llvm::Type *> ArgumentTypes(FunctionType->param_begin(),
+                                            FunctionType->param_end());
+    ArgumentTypes.push_back(llvm::Type::getInt32Ty(Module->getContext()));
+
+    // Create new function.
+    llvm::FunctionType *NewFunctionType = llvm::FunctionType::get(
+        Function->getReturnType(), ArgumentTypes, false);
+    llvm::Function *NewFunction = llvm::Function::Create(
+        NewFunctionType, Function->getLinkage(), Function->getName(), Module);
+    NewFunction->takeName(Function);
+
+    // Map function arguments.
+    llvm::ValueToValueMapTy VV;
+    llvm::Function::arg_iterator OldArg;
+    llvm::Function::arg_iterator NewArg;
+    for (OldArg = Function->arg_begin(), NewArg = NewFunction->arg_begin();
+         OldArg != Function->arg_end(); NewArg++, OldArg++) {
+      NewArg->takeName(&*OldArg);
+      VV[&*OldArg] = &*NewArg;
+    }
+
+    // Clone function.
+    llvm::SmallVector<llvm::ReturnInst *, 1> RI;
+    CloneFunctionIntoAbs(NewFunction, Function, VV, RI);
+    FunctionsToErase.push_back(Function);
+
+    // Replace uses of the global offset variables with the new arguments.
+    NewArg->setName("work_dim");
+    // replaceScalarGlobalVar(Module, "_work_dim", (&*NewArg++));
+
+    // TODO: What if get_work_dim() is called from a non-kernel function?
+  }
+
+  for (auto F : FunctionsToErase) {
+    F->eraseFromParent();
+  }
 }
 
 int findLibDevice(char LibDevicePath[PATH_MAX], const char *Arch) {
@@ -609,19 +675,22 @@ int findLibDevice(char LibDevicePath[PATH_MAX], const char *Arch) {
 // This would remove this runtime dependency on the CUDA toolkit.
 // Had some issues with the earlier pocl LLVM passes crashing on the libdevice
 // code - needs more investigation.
-void linkLibDevice(llvm::Module *Module, const char *KernelName,
-                   const char *LibDevicePath) {
+int linkLibDevice(llvm::Module *Module, const char *LibDevicePath) {
   auto Buffer = llvm::MemoryBuffer::getFile(LibDevicePath);
-  if (!Buffer)
-    POCL_ABORT("[CUDA] failed to open libdevice library file\n");
+  if (!Buffer) {
+    POCL_MSG_ERR("[CUDA] failed to open libdevice library file\n");
+    return -1;
+  }
 
   POCL_MSG_PRINT_INFO("loading libdevice from '%s'\n", LibDevicePath);
 
   // Load libdevice bitcode library.
   llvm::Expected<std::unique_ptr<llvm::Module>> LibDeviceModule =
       parseBitcodeFile(Buffer->get()->getMemBufferRef(), Module->getContext());
-  if (!LibDeviceModule)
-    POCL_ABORT("[CUDA] failed to load libdevice bitcode\n");
+  if (!LibDeviceModule) {
+    POCL_MSG_ERR("[CUDA] failed to load libdevice bitcode\n");
+    return -1;
+  }
 
   // Fix triple and data-layout of libdevice module.
   (*LibDeviceModule)->setTargetTriple(Module->getTargetTriple());
@@ -629,18 +698,12 @@ void linkLibDevice(llvm::Module *Module, const char *KernelName,
 
   // Link libdevice into module.
   llvm::Linker Linker(*Module);
-  if (Linker.linkInModule(std::move(LibDeviceModule.get()))) {
-    POCL_ABORT("[CUDA] failed to link to libdevice\n");
+  if (Linker.linkInModule(std::move(LibDeviceModule.get()),
+                          llvm::Linker::Flags::LinkOnlyNeeded)) {
+    POCL_MSG_ERR("[CUDA] failed to link to libdevice\n");
+    return -1;
   }
 
-  llvm::legacy::PassManager Passes;
-
-  // Run internalize to mark all non-kernel functions as internal.
-  auto PreserveKernel = [=](const llvm::GlobalValue &GV) {
-    return GV.getName() == KernelName;
-  };
-  Passes.add(llvm::createInternalizePass(PreserveKernel));
-
   // Add NVVM reflect module flags to set math options.
   // TODO: Determine correct FTZ value from frontend compiler options.
   llvm::LLVMContext &Context = Module->getContext();
@@ -654,28 +717,36 @@ void linkLibDevice(llvm::Module *Module, const char *KernelName,
       llvm::MDNode::get(Context, {FourMD, NameMD, OneMD});
   Module->addModuleFlag(ReflectFlag);
 
-  // Run optimization passes to clean up unused functions etc.
-  llvm::PassManagerBuilder Builder;
-  Builder.OptLevel = 3;
-  Builder.SizeLevel = 0;
-  Builder.populateModulePassManager(Passes);
+  // Run internalize to mark all non-kernel functions as internal.
+  auto PreserveKernel = [=](const llvm::GlobalValue &GV) {
+    const llvm::Function *F = llvm::dyn_cast<llvm::Function>(&GV);
+    return (F != nullptr && pocl::isKernelToProcess(*F));
+  };
 
+#if LLVM_MAJOR < 17
+  llvm::legacy::PassManager Passes;
+  Passes.add(llvm::createInternalizePass(PreserveKernel));
+  // run the InternalizePass
   Passes.run(*Module);
+#else
+  internalizeModule(*Module, PreserveKernel);
+#endif
+
+  // Run optimization passes to clean up unused functions etc.
+  populateModulePM(nullptr, Module, 3, 0);
+
+  return 0;
 }
 
 // This transformation replaces each pointer argument in the specific address
 // space with an integer offset, and then inserts the necessary GEP+BitCast
 // instructions to calculate the new pointers from the provided base global
 // variable.
-void convertPtrArgsToOffsets(llvm::Module *Module, const char *KernelName,
+bool convertPtrArgsToOffsets(llvm::Module *Module, llvm::Function *Function,
                              unsigned AddrSpace, llvm::GlobalVariable *Base) {
 
   llvm::LLVMContext &Context = Module->getContext();
 
-  llvm::Function *Function = Module->getFunction(KernelName);
-  if (!Function)
-    POCL_ABORT("[CUDA] ptx-gen: kernel function not found in module\n");
-
   // Argument info for creating new function.
   std::vector<llvm::Argument *> Arguments;
   std::vector<llvm::Type *> ArgumentTypes;
@@ -724,7 +795,7 @@ void convertPtrArgsToOffsets(llvm::Module *Module, const char *KernelName,
   }
 
   if (!NeedsArgOffsets)
-    return;
+    return false;
 
   // Create new function with offsets instead of local memory pointers.
   llvm::FunctionType *NewFunctionType =
@@ -758,12 +829,14 @@ void convertPtrArgsToOffsets(llvm::Module *Module, const char *KernelName,
     Pair.second->insertAfter(Pair.first);
   }
 
-  Function->eraseFromParent();
+  return true;
 }
 
 // CUDA doesn't allow constant pointer arguments, so we have to convert them to
 // offsets and manually add them to a global variable base pointer.
-void fixConstantMemArgs(llvm::Module *Module, const char *KernelName) {
+void fixConstantMemArgs(llvm::Module *Module) {
+
+  llvm::SmallVector<llvm::Function *, 8> FunctionsToErase;
 
   // Calculate total size of automatic constant allocations.
   size_t TotalAutoConstantSize = 0;
@@ -783,14 +856,24 @@ void fixConstantMemArgs(llvm::Module *Module, const char *KernelName) {
       NULL, "_constant_memory_region_",
       NULL, llvm::GlobalValue::NotThreadLocal, 4, false);
 
-  convertPtrArgsToOffsets(Module, KernelName, 4, ConstantMemBase);
+  for (auto &FI : Module->functions()) {
+    if (!pocl::isKernelToProcess(FI))
+      continue;
+    if (convertPtrArgsToOffsets(Module, &FI, 4, ConstantMemBase))
+      FunctionsToErase.push_back(&FI);
+  }
+  for (auto F : FunctionsToErase) {
+    F->eraseFromParent();
+  }
 }
 
 // CUDA doesn't allow multiple local memory arguments or automatic variables, so
 // we have to create a single global variable for local memory allocations, and
 // then manually add offsets to it to get each individual local memory
 // allocation.
-void fixLocalMemArgs(llvm::Module *Module, const char *KernelName) {
+void fixLocalMemArgs(llvm::Module *Module) {
+
+  llvm::SmallVector<llvm::Function *, 8> FunctionsToErase;
 
   // Create global variable for local memory allocations.
   llvm::Type *ByteArrayType =
@@ -800,7 +883,16 @@ void fixLocalMemArgs(llvm::Module *Module, const char *KernelName) {
       "_shared_memory_region_", NULL, llvm::GlobalValue::NotThreadLocal, 3,
       false);
 
-  convertPtrArgsToOffsets(Module, KernelName, 3, SharedMemBase);
+  for (auto &FI : Module->functions()) {
+    if (!pocl::isKernelToProcess(FI))
+      continue;
+    if (convertPtrArgsToOffsets(Module, &FI, 3, SharedMemBase))
+      FunctionsToErase.push_back(&FI);
+  }
+
+  for (auto F : FunctionsToErase) {
+    F->eraseFromParent();
+  }
 }
 
 // Map kernel math functions onto the corresponding CUDA libdevice functions.
@@ -868,12 +960,16 @@ void mapLibDeviceCalls(llvm::Module *Module) {
 
     {"frexp", "__nv_frexp"},
     {"frexpf", "__nv_frexpf"},
+    {"llvm.frexp.f64.i32", "frexp_f64_i32"},
+    {"llvm.frexp.f32.i32", "frexpf_f32_i32"},
 
     {"tgamma", "__nv_tgamma"},
     {"tgammaf", "__nv_tgammaf"},
 
     {"ldexp", "__nv_ldexp"},
     {"ldexpf", "__nv_ldexpf"},
+    {"llvm.ldexp.f64.i32", "__nv_ldexp"},
+    {"llvm.ldexp.f32.i32", "__nv_ldexpf"},
 
     {"modf", "__nv_modf"},
     {"modff", "__nv_modff"},
@@ -899,14 +995,9 @@ void mapLibDeviceCalls(llvm::Module *Module) {
       if (Call) {
         // Create function declaration for libdevice version.
         llvm::FunctionType *FunctionType = Function->getFunctionType();
-#ifdef LLVM_OLDER_THAN_9_0
-        llvm::Constant *LibDeviceFunction = Module->getOrInsertFunction(
-            Entry.LibDeviceFunctionName, FunctionType);
-#else
         llvm::FunctionCallee FC = Module->getOrInsertFunction(
             Entry.LibDeviceFunctionName, FunctionType);
         llvm::Function *LibDeviceFunction = llvm::cast<llvm::Function>(FC.getCallee());
-#endif
         // Replace function with libdevice version.
         std::vector<llvm::Value *> Args(Call->arg_begin(), Call->arg_end());
         llvm::CallInst *NewCall =
@@ -921,37 +1012,27 @@ void mapLibDeviceCalls(llvm::Module *Module) {
   }
 }
 
-int pocl_cuda_get_ptr_arg_alignment(const char *BitcodeFilename,
-                                    const char *KernelName,
-                                    size_t *Alignments) {
-  // Create buffer for bitcode file.
-  llvm::ErrorOr<std::unique_ptr<llvm::MemoryBuffer>> Buffer =
-      llvm::MemoryBuffer::getFile(BitcodeFilename);
-  if (!Buffer) {
-    POCL_MSG_ERR("[CUDA] ptx-gen: failed to open bitcode file\n");
-    return 1;
-  }
-
+static int getPtrArgAlignment(llvm::Module *Module, llvm::Function *Kernel,
+                              std::vector<size_t> &AlignmentVec) {
   // Load the LLVM bitcode module.
-  llvm::LLVMContext Context;
-  llvm::Expected<std::unique_ptr<llvm::Module>> Module =
-      parseBitcodeFile(Buffer->get()->getMemBufferRef(), Context);
   if (!Module) {
     POCL_MSG_ERR("[CUDA] ptx-gen: failed to load bitcode\n");
-    return 1;
+    return -1;
   }
 
   // Get kernel function.
-  llvm::Function *Kernel = (*Module)->getFunction(KernelName);
-  if (!Kernel)
-    POCL_ABORT("[CUDA] kernel function not found in module\n");
+  if (!Kernel) {
+    POCL_MSG_ERR("[CUDA] kernel function not found in module\n");
+    return -1;
+  }
 
   // Calculate alignment for each argument.
-  const llvm::DataLayout &DL = (*Module)->getDataLayout();
+  const llvm::DataLayout &DL = Module->getDataLayout();
   for (auto &Arg : Kernel->args()) {
     unsigned i = Arg.getArgNo();
     llvm::Type *Type = Arg.getType();
-    Alignments[i] = 0;
+    AlignmentVec.push_back(0);
+    assert(i < AlignmentVec.size());
     // TODO test this
     if (Type->isPointerTy()) {
       // try to figure out alignment from uses
@@ -960,24 +1041,92 @@ int pocl_cuda_get_ptr_arg_alignment(const char *BitcodeFilename,
                 llvm::dyn_cast<llvm::GetElementPtrInst>(U)) {
           for (auto UU : GEP->users()) {
             if (llvm::StoreInst *SI = llvm::dyn_cast<llvm::StoreInst>(UU)) {
-              Alignments[i] = SI->getAlign().value();
+              AlignmentVec[i] = SI->getAlign().value();
               break;
             }
             if (llvm::LoadInst *LI = llvm::dyn_cast<llvm::LoadInst>(UU)) {
-              Alignments[i] = LI->getAlign().value();
+              AlignmentVec[i] = LI->getAlign().value();
               break;
             }
           }
-          if (Alignments[i])
+          if (AlignmentVec[i])
             break;
         }
       }
-      if (Alignments[i] == 0)
-        Alignments[i] = MAX_EXTENDED_ALIGNMENT;
+      if (AlignmentVec[i] == 0)
+        AlignmentVec[i] = MAX_EXTENDED_ALIGNMENT;
+      //      POCL_MSG_WARN("V1 ||||| Argument %u : ALIGN %zu \n", i,
+      //      AlignmentVec[i]);
     } else {
-      Alignments[i] = Arg.getParamAlignment();
+#ifdef LLVM_OLDER_THAN_16_0
+      AlignmentVec[i] = Arg.getParamAlignment();
+#else
+      if (Arg.getType()->isPointerTy())
+        AlignmentVec[i] = Arg.getParamAlign().valueOrOne().value();
+      else
+        AlignmentVec[i] = DL.getTypeAllocSize(Arg.getType());
+#endif
+      //      POCL_MSG_WARN("V2 ||||| Argument %u : ALIGN %zu \n", i,
+      //      AlignmentVec[i]);
     }
   }
 
   return 0;
 }
+
+void createAlignmentMap(llvm::Module *Module, AlignmentMapT *AlignmentMap) {
+  for (auto &FI : Module->functions()) {
+    if (!pocl::isKernelToProcess(FI))
+      continue;
+    std::string Name = FI.getName().str();
+    if (AlignmentMap->find(Name) != AlignmentMap->end())
+      continue;
+    std::vector<size_t> &AVec = (*AlignmentMap)[Name];
+    AVec.reserve(4);
+    if (getPtrArgAlignment(Module, &FI, AVec) != 0)
+      POCL_MSG_ERR("can't figure out alignments for kernel %s", Name.c_str());
+  }
+}
+
+int pocl_cuda_create_alignments(void *llvm_module, void **AlignmentMapPtr) {
+  llvm::Module *Module = (llvm::Module *)llvm_module;
+  if (!Module) {
+    POCL_MSG_ERR("[CUDA] ptx-gen: failed to load bitcode\n");
+    return -1;
+  }
+  bool VerifyMod =
+      pocl_get_bool_option("POCL_LLVM_VERIFY", LLVM_VERIFY_MODULE_DEFAULT);
+
+  AlignmentMapT *A = new AlignmentMapT;
+  assert(*AlignmentMapPtr == nullptr);
+  *AlignmentMapPtr = A;
+  createAlignmentMap(Module, A);
+
+  if (VerifyMod && !verifyModule(Module, "getAlignmentMap"))
+    return -1;
+  return 0;
+}
+
+void pocl_cuda_destroy_alignments(void *llvm_module, void *AlignmentMapPtr) {
+  if (AlignmentMapPtr != nullptr) {
+    AlignmentMapT *A = (AlignmentMapT *)(AlignmentMapPtr);
+    delete A;
+  }
+}
+
+int pocl_cuda_get_ptr_arg_alignment(void *LLVM_IR, const char *KernelName,
+                                    size_t *Alignments, void *AlignmentMapPtr) {
+  AlignmentMapT *AMap = (AlignmentMapT *)AlignmentMapPtr;
+  assert(AMap != nullptr);
+
+  std::string Name(KernelName);
+  if (AMap->find(Name) == AMap->end()) {
+    POCL_MSG_ERR(
+        "pocl_cuda_get_ptr_arg_alignment: kernel not found in module\n");
+    return 1;
+  }
+  std::vector<size_t> &AVec = AMap->at(Name);
+  //  POCL_MSG_WARN("AVEC SIZE: %zu\n", AVec.size());
+  std::memcpy(Alignments, AVec.data(), sizeof(size_t) * AVec.size());
+  return 0;
+}
