@@ -215,6 +215,7 @@ typedef struct pocl_hsa_device_data_s {
   void *printf_buffer;
   uint32_t *printf_write_pos;
 
+  cl_bool available;
 } pocl_hsa_device_data_t;
 
 void pocl_hsa_compile_kernel_hsail (_cl_command_node *cmd, cl_kernel kernel,
@@ -567,6 +568,8 @@ init_dev_data (cl_device_id dev, int count)
   dev->data = d;
   d->device = dev;
   d->hw_schedulers = 3;
+  d->available = CL_TRUE;
+  dev->available = &d->available;
   POCL_INIT_LOCK (d->pocl_hsa_compilation_lock);
 
   /* Before the first HSA device, re-init the runtime. */
@@ -704,6 +707,19 @@ pocl_hsa_init (unsigned j, cl_device_id dev, const char *parameters)
   HSA_CHECK(hsa_agent_get_info (agent, HSA_AGENT_INFO_NAME, dev->long_name));
   get_hsa_device_features (dev->long_name, dev);
 
+  if (dev->llvm_target_triplet != NULL) {
+    if (strcmp(dev->llvm_target_triplet, "hsail64") == 0) {
+      dev->kernellib_name = "kernel-hsail64";
+      dev->kernellib_fallback_name = NULL;
+      dev->kernellib_subdir = "hsail64";
+    }
+    if (strcmp(dev->llvm_target_triplet, "amdgcn--amdhsa") == 0) {
+      dev->kernellib_name = "kernel-amdgcn--amdhsa";
+      dev->kernellib_fallback_name = NULL;
+      dev->kernellib_subdir = "amdgcn";
+    }
+  }
+
   dev->type = CL_DEVICE_TYPE_GPU;
 
   // Enable when it's actually implemented AND if supported by
