@@ -63,9 +63,7 @@ struct event_data {
 };
 
 struct data {
-  /* Currently loaded kernel. */
-  cl_kernel current_kernel;
-  volatile uint64_t total_cmd_exec_time;
+  cl_bool available;
 };
 
 void
@@ -73,7 +71,7 @@ pocl_pthread_init_device_ops(struct pocl_device_ops *ops)
 {
   pocl_basic_init_device_ops(ops);
 
-  ops->device_name = "pthread";
+  ops->device_name = "cpu";
 
   /* implementation that differs from basic */
   ops->probe = pocl_pthread_probe;
@@ -101,7 +99,7 @@ char *
 pocl_pthread_build_hash (cl_device_id device)
 {
   char* res = calloc(1000, sizeof(char));
-  snprintf (res, 1000, "pthread-%s-%s", HOST_DEVICE_BUILD_HASH,
+  snprintf (res, 1000, "cpu-%s-%s", HOST_DEVICE_BUILD_HASH,
             device->llvm_cpu);
   return res;
 }
@@ -110,6 +108,11 @@ unsigned int
 pocl_pthread_probe (struct pocl_device_ops *ops)
 {
   int env_count = pocl_device_get_env_count(ops->device_name);
+
+  /* for backwards compatibility */
+  if (env_count <= 0)
+    env_count = pocl_device_get_env_count("pthread");
+
   /* Env was not specified, default behavior was to use 1 pthread device */
   if (env_count < 0)
     return 1;
@@ -127,6 +130,8 @@ static int scheduler_initialized = 0;
 cl_int
 pocl_pthread_init (unsigned j, cl_device_id device, const char* parameters)
 {
+  /* TODO: call pocl_basic_init() to avoid duplicating a lot of the
+     initializations here. */
   struct data *d;
   int err;
 
@@ -134,10 +139,24 @@ pocl_pthread_init (unsigned j, cl_device_id device, const char* parameters)
   if (d == NULL)
     return CL_OUT_OF_HOST_MEMORY;
 
-  d->current_kernel = NULL;
   device->data = d;
+  d->available = CL_TRUE;
+  device->available = &d->available;
 
   pocl_init_default_device_infos (device);
+
+  if (strstr (HOST_DEVICE_EXTENSIONS, "cl_khr_subgroup") != NULL)
+    {
+      /* In reality there is no independent SG progress implemented in this
+         version because we can only have one SG in flight at a time, but it's
+         a corner case which allows us to advertise it for full CTS compliance.
+       */
+      device->sub_group_independent_forward_progress = CL_TRUE;
+
+      /* Just an arbitrary number here based on assumption of SG size 32. */
+      device->max_num_sub_groups = device->max_work_group_size / 32;
+    }
+
   /* 0 is the host memory shared with all drivers that use it */
   device->global_mem_id = 0;
 
@@ -146,6 +165,8 @@ pocl_pthread_init (unsigned j, cl_device_id device, const char* parameters)
 
 #if (HOST_DEVICE_CL_VERSION_MAJOR >= 3)
   device->features = HOST_DEVICE_FEATURES_30;
+  device->run_program_scope_variables_pass = CL_TRUE;
+  device->generic_as_support = CL_TRUE;
 
   pocl_setup_opencl_c_with_version (device, CL_TRUE);
   pocl_setup_features_with_version (device);
@@ -185,8 +206,35 @@ pocl_pthread_init (unsigned j, cl_device_id device, const char* parameters)
   device->svm_caps = CL_DEVICE_SVM_COARSE_GRAIN_BUFFER
                      | CL_DEVICE_SVM_FINE_GRAIN_BUFFER
                      | CL_DEVICE_SVM_ATOMICS;
+
+  if (strstr (HOST_DEVICE_EXTENSIONS, "cl_ext_float_atomics")
+      != NULL) {
+    device->single_fp_atomic_caps = device->double_fp_atomic_caps =
+      CL_DEVICE_GLOBAL_FP_ATOMIC_LOAD_STORE_EXT |
+      CL_DEVICE_GLOBAL_FP_ATOMIC_ADD_EXT |
+      CL_DEVICE_GLOBAL_FP_ATOMIC_MIN_MAX_EXT |
+      CL_DEVICE_LOCAL_FP_ATOMIC_LOAD_STORE_EXT |
+      CL_DEVICE_LOCAL_FP_ATOMIC_ADD_EXT |
+      CL_DEVICE_LOCAL_FP_ATOMIC_MIN_MAX_EXT;
+  }
+
 #endif
 
+  if (strstr (HOST_DEVICE_EXTENSIONS, "cl_intel_unified_shared_memory")
+      != NULL)
+    {
+      device->host_usm_capabs = CL_UNIFIED_SHARED_MEMORY_ACCESS_INTEL
+                                | CL_UNIFIED_SHARED_MEMORY_ATOMIC_ACCESS_INTEL;
+
+      device->device_usm_capabs
+          = CL_UNIFIED_SHARED_MEMORY_ACCESS_INTEL
+            | CL_UNIFIED_SHARED_MEMORY_ATOMIC_ACCESS_INTEL;
+
+      device->single_shared_usm_capabs
+          = CL_UNIFIED_SHARED_MEMORY_ACCESS_INTEL
+            | CL_UNIFIED_SHARED_MEMORY_ATOMIC_ACCESS_INTEL;
+    }
+
   /* hwloc probes OpenCL device info at its initialization in case
      the OpenCL extension is enabled. This causes to printout 
      an unimplemented property error because hwloc is used to
@@ -196,20 +244,32 @@ pocl_pthread_init (unsigned j, cl_device_id device, const char* parameters)
   if (err)
     return CL_INVALID_DEVICE;
 
-  /* device->max_compute_units was set up by topology_detect,
-   * but if the user requests, lower it */
+  /* if hwloc/topology detection failed, use a fixed maximum */
   int fallback = (device->max_compute_units == 0) ? FALLBACK_MAX_THREAD_COUNT
                                                   : device->max_compute_units;
-  int max_thr = pocl_get_int_option ("POCL_MAX_PTHREAD_COUNT", fallback);
+
+  /* device->max_compute_units was set up by topology_detect,
+   * but if the user requests, lower it */
+
+  /* old env variable */
+  int max_threads = pocl_get_int_option ("POCL_MAX_PTHREAD_COUNT", 0);
+
+  if (max_threads <= 0)
+    max_threads = pocl_get_int_option ("POCL_CPU_MAX_CU_COUNT", fallback);
+
+  /* old env variable */
+  int min_threads = pocl_get_int_option ("POCL_PTHREAD_MIN_THREADS", 0);
+  if (min_threads <= 0)
+    min_threads = pocl_get_int_option ("POCL_CPU_MIN_CU_COUNT", 1);
 
   device->max_compute_units
-      = max ((unsigned)max_thr,
-             (unsigned)pocl_get_int_option ("POCL_PTHREAD_MIN_THREADS", 1));
+      = max ((unsigned)max_threads, (unsigned)min_threads);
 
   pocl_cpuinfo_detect_device_info(device);
   pocl_set_buffer_image_limits(device);
 
-  fix_local_mem_size (device);
+  device->local_mem_size = pocl_get_int_option ("POCL_CPU_LOCAL_MEM_SIZE",
+                                                device->local_mem_size);
 
   /* in case hwloc doesn't provide a PCI ID, let's generate
      a vendor id that hopefully is unique across vendors. */
@@ -259,7 +319,7 @@ pocl_pthread_uninit (unsigned j, cl_device_id device)
 }
 
 cl_int
-pocl_pthread_reinit (unsigned j, cl_device_id device)
+pocl_pthread_reinit (unsigned j, cl_device_id device, const char *parameters)
 {
   struct data *d;
 
@@ -267,7 +327,6 @@ pocl_pthread_reinit (unsigned j, cl_device_id device)
   if (d == NULL)
     return CL_OUT_OF_HOST_MEMORY;
 
-  d->current_kernel = NULL;
   device->data = d;
 
   cl_int ret = CL_SUCCESS;
