@@ -34,6 +34,7 @@
 
 #include "builtin_kernels.hh"
 #include "common.h"
+#include "common_driver.h"
 #include "pocl-pthread.h"
 #include "pocl-pthread_scheduler.h"
 #include "pocl-pthread_utils.h"
@@ -130,6 +131,9 @@ pthread_scheduler_init (cl_device_id device)
       PTHREAD_CHECK (pthread_create (&scheduler.thread_pool[i].thread, NULL,
                                      pocl_pthread_driver_thread,
                                      (void *)&scheduler.thread_pool[i]));
+#if defined(__linux__) && defined(__x86_64__)
+      pocl_ignore_sigfpe_for_thread (scheduler.thread_pool[i].thread);
+#endif
     }
 
   PTHREAD_CHECK2 (PTHREAD_BARRIER_SERIAL_THREAD,
@@ -376,11 +380,12 @@ pocl_pthread_prepare_kernel (void *data, _cl_command_node *cmd)
   kernel_run_command *run_cmd;
   cl_kernel kernel = cmd->command.run.kernel;
   struct pocl_context *pc = &cmd->command.run.pc;
+  cl_program program = kernel->program;
+  cl_uint dev_i = cmd->program_device_i;
 
-  char *saved_name = NULL;
-  pocl_sanitize_builtin_kernel_name (kernel, &saved_name);
-  pocl_check_kernel_dlhandle_cache (cmd, 1, 1);
-  pocl_restore_builtin_kernel_name (kernel, saved_name);
+  /* initialize the program gvars if required */
+  pocl_driver_build_gvar_init_kernel (program, dev_i, cmd->device,
+                                      pocl_cpu_gvar_init_callback);
 
   size_t num_groups = pc->num_groups[0] * pc->num_groups[1] * pc->num_groups[2];
 
@@ -394,7 +399,10 @@ pocl_pthread_prepare_kernel (void *data, _cl_command_node *cmd)
       return;
     }
 
-  pocl_check_kernel_dlhandle_cache (cmd, 1, 1);
+  char *saved_name = NULL;
+  pocl_sanitize_builtin_kernel_name (kernel, &saved_name);
+  pocl_check_kernel_dlhandle_cache (cmd, CL_TRUE, CL_TRUE);
+  pocl_restore_builtin_kernel_name (kernel, saved_name);
 
   run_cmd = new_kernel_run_command ();
   run_cmd->data = data;
@@ -405,6 +413,7 @@ pocl_pthread_prepare_kernel (void *data, _cl_command_node *cmd)
   run_cmd->pc.printf_buffer = NULL;
   run_cmd->pc.printf_buffer_capacity = scheduler.printf_buf_size;
   run_cmd->pc.printf_buffer_position = NULL;
+  run_cmd->pc.global_var_buffer = program->gvar_storage[dev_i];
   run_cmd->remaining_wgs = num_groups;
   run_cmd->wgs_dealt = 0;
   run_cmd->workgroup = cmd->command.run.wg;
