@@ -212,10 +212,20 @@ public:
     if (machine_->isLittleEndian()) {
       dev->endian_little = CL_TRUE;
       dev->llvm_target_triplet = "tcele-tut-llvm";
+      dev->kernellib_name = "kernel-tcele-tut-llvm";
     } else {
       dev->endian_little = CL_FALSE;
       dev->llvm_target_triplet = "tce-tut-llvm";
+      dev->kernellib_name = "kernel-tce-tut-llvm";
+      /* with fp-contract=on we get calls to fma with processors which do not
+       * have fma instructions. These ruin the performance.
+       *
+       * default fp-contract is "on" which means "enable if enabled by a pragma".
+       */
+      dev->llvm_fp_contract_mode = "off";
     }
+    dev->kernellib_fallback_name = NULL;
+    dev->kernellib_subdir = "tce";
 
 #if defined(WORDS_BIGENDIAN) && WORDS_BIGENDIAN == 1
     needsByteSwap = ((dev->endian_little == CL_TRUE) ? true : false);
@@ -612,7 +622,6 @@ private:
 static const char *tta_native_device_aux_funcs[] = {"_pocl_memcpy_1",
                                                     "_pocl_memcpy_4", NULL};
 
-static const char *tce_params[256];
 
 cl_int
 pocl_ttasim_init (unsigned j, cl_device_id dev, const char* parameters)
@@ -625,7 +634,6 @@ pocl_ttasim_init (unsigned j, cl_device_id dev, const char* parameters)
   dev->max_compute_units = 1;
   dev->max_work_item_dimensions = 3;
   dev->device_side_printf = 0;
-  tce_params[j] = parameters;
 
   int max_wg
       = pocl_get_int_option ("POCL_MAX_WORK_GROUP_SIZE", DEFAULT_WG_SIZE);
@@ -679,13 +687,12 @@ pocl_ttasim_init (unsigned j, cl_device_id dev, const char* parameters)
      }
   dev->builtins_sources_path = "builtins.cl";
 
-  dev->available = CL_TRUE;
 #ifdef ENABLE_LLVM
   dev->compiler_available = CL_TRUE;
   dev->linker_available = CL_TRUE;
 #endif
   dev->spmd = CL_FALSE;
-  dev->workgroup_pass = CL_TRUE;
+  dev->run_workgroup_pass = CL_TRUE;
   dev->execution_capabilities = CL_EXEC_KERNEL;
   dev->queue_properties = CL_QUEUE_PROFILING_ENABLE;
   dev->vendor = "TTA-Based Co-design Environment";
@@ -718,6 +725,7 @@ pocl_ttasim_init (unsigned j, cl_device_id dev, const char* parameters)
 
   dev->data = new TTASimDevice(dev, parameters);
 
+  dev->available = &((TTASimDevice *)dev->data)->available;
   dev->arg_buffer_launcher = CL_TRUE;
   dev->grid_launcher = CL_FALSE;
   dev->device_aux_functions = tta_native_device_aux_funcs;
@@ -749,9 +757,10 @@ pocl_ttasim_uninit (unsigned j, cl_device_id device)
   return CL_SUCCESS;
 }
 
-cl_int pocl_ttasim_reinit(unsigned j, cl_device_id device) {
+cl_int pocl_ttasim_reinit(unsigned j, cl_device_id device,
+                          const char *parameters) {
   POCL_MSG_PRINT_TCE("DEV REINIT \n");
-  device->data = new TTASimDevice(device, tce_params[j]);
+  device->data = new TTASimDevice(device, parameters);
   return CL_SUCCESS;
 }
 
