@@ -30,6 +30,7 @@
 #include "pocl_binary.h"
 #include "pocl_cache.h"
 #include "pocl_file_util.h"
+#include "pocl_llvm.h"
 
 #include <sys/stat.h>
 #include <dirent.h>
@@ -53,10 +54,12 @@
                           is already in cl_kernel_arg_address_qualifier);
                           add has_arg_metadata & kernel attributes */
 /* changes for version 8: compilation parameters are stored in module metadata
-                          */
+ * changes for version 9: support other than "program.bc" files in root dir
+ * changes for version 10: support program scope variables
+ * changes for version 11: support extra subgroup & workgroup metadata */
 
-#define FIRST_SUPPORTED_POCLCC_VERSION 8
-#define POCLCC_VERSION 8
+#define FIRST_SUPPORTED_POCLCC_VERSION 9
+#define POCLCC_VERSION 11
 
 /* pocl binary structures */
 
@@ -67,6 +70,9 @@
  * 4) files are written as two strings: | uint32_t | relative filename | uint32_t | content |
  */
 
+#define POCL_KERNEL_HAS_WORKG_META (1 << 1)
+#define POCL_KERNEL_HAS_SUBG_META (1 << 2)
+
 typedef struct pocl_binary_kernel_s
 {
   /* the first 3 fields are sizes in bytes of the data pieces that follow
@@ -92,10 +98,24 @@ typedef struct pocl_binary_kernel_s
   /* number of kernel local variables */
   uint32_t num_locals;
 
+  /* per-device subgroup metadata */
+  uint32_t max_subgroups;
+  uint32_t compile_subgroups;
+
+  /* per-device workgroup metadata */
+  uint32_t max_wg_size;
+  uint32_t preferred_wg_multiple;
+  uint32_t local_mem_size;
+  uint32_t private_mem_size;
+  uint32_t spill_mem_size;
+
   /* required work-group size */
   uint64_t reqd_wg_size[OPENCL_MAX_DIMENSION];
 
-  uint64_t has_arg_metadata;
+  /* Stores: cl_bitfield kernel->meta->has_arg_metadata */
+  uint32_t has_arg_metadata;
+  /* Stores: extra flags (POCL_KERNEL_HAS) */
+  uint32_t flags;
 
   uint32_t sizeof_attributes;
   char* attributes;
@@ -117,11 +137,18 @@ typedef struct pocl_binary_s
   uint32_t num_kernels;
   /* various flags */
   uint64_t flags;
+  /* number of files/directories serialized
+   * from the main cache directory */
+  uint32_t root_entries;
   /* program->build_hash[device_i], required to restore files into pocl cache */
   SHA1_digest_t program_build_hash;
+  /* bytes of storage required for program scope vars */
+  uint64_t program_scope_var_bytes;
 } pocl_binary;
 
+/* pocl_binary flags  */
 #define POCL_BINARY_FLAG_FLUSH_DENORMS (1 << 0)
+#define POCL_BINARY_HAS_PROG_SCOPE_VARS (1 << 1)
 
 #define TO_LE(x)                                \
   ((sizeof(x) == 8) ? htole64((uint64_t)x) :    \
@@ -207,8 +234,13 @@ read_header(pocl_binary *b, const unsigned char *buffer)
   BUFFER_READ(b->version, uint32_t);
   BUFFER_READ(b->num_kernels, uint32_t);
   BUFFER_READ (b->flags, uint64_t);
+  BUFFER_READ(b->root_entries, uint32_t);
   memcpy(b->program_build_hash, buffer, sizeof(SHA1_digest_t));
   buffer += sizeof(SHA1_digest_t);
+  if (b->flags & POCL_BINARY_HAS_PROG_SCOPE_VARS)
+    {
+      BUFFER_READ (b->program_scope_var_bytes, uint64_t);
+    }
   return (unsigned char*)buffer;
 }
 
@@ -253,7 +285,7 @@ check_binary(cl_device_id device, const unsigned char *binary)
   uint64_t dev_id = pocl_binary_get_device_id(device);
   if (dev_id != b.device_id)
     {
-      POCL_MSG_WARN ("PoCLBinary device id mismatch, DEVICE: %lx, BINARY: %lx\n",
+      POCL_MSG_WARN ("PoCLBinary device id mismatch, DEVICE: %" PRIX64 ", BINARY: %" PRIX64 "\n",
                       dev_id, b.device_id);
       return NULL;
     }
@@ -311,7 +343,8 @@ recursively_serialize_path (char* path,
                             unsigned char* buffer)
 {
   struct stat st;
-  stat (path, &st);
+  if (stat (path, &st) != 0)
+    return buffer;
 
   if (S_ISREG (st.st_mode))
     buffer = serialize_file (path, basedir_offset, buffer);
@@ -320,9 +353,9 @@ recursively_serialize_path (char* path,
     {
       DIR *d;
       struct dirent *entry;
-      char subpath[POCL_FILENAME_LENGTH];
+      char subpath[POCL_MAX_PATHNAME_LENGTH];
 
-      strncpy (subpath, path, POCL_FILENAME_LENGTH-1);
+      strncpy (subpath, path, POCL_MAX_PATHNAME_LENGTH - 1);
       char* p = subpath + strlen(subpath);
       *p++ = '/';
       d = opendir (path);
@@ -347,8 +380,8 @@ serialize_kernel_cachedir (cl_program program,
                            unsigned device_i,
                            unsigned char* buffer)
 {
-  char path[POCL_FILENAME_LENGTH];
-  char basedir[POCL_FILENAME_LENGTH];
+  char path[POCL_MAX_PATHNAME_LENGTH];
+  char basedir[POCL_MAX_PATHNAME_LENGTH];
 
   pocl_cache_program_path (basedir, program, device_i);
   size_t basedir_len = strlen (basedir);
@@ -358,8 +391,6 @@ serialize_kernel_cachedir (cl_program program,
                        kernel_name, path);
   if (pocl_exists (path))
     buffer = recursively_serialize_path (path, basedir_len, buffer);
-  else
-    POCL_MSG_ERR ("CAN't serialize %s - doesn't exist \n", path);
 
   return buffer;
 }
@@ -395,7 +426,57 @@ pocl_binary_serialize_kernel_to_buffer(cl_program program,
 
   uint32_t attrlen = meta->attributes ? strlen (meta->attributes) : 0;
   BUFFER_STORE_STR2(meta->attributes, attrlen);
-  BUFFER_STORE(meta->has_arg_metadata, uint64_t);
+  uint32_t has_meta = meta->has_arg_metadata;
+  BUFFER_STORE (has_meta, uint32_t);
+  uint32_t flags = 0;
+  if ((meta->max_subgroups && meta->max_subgroups[device_i])
+      || (meta->compile_subgroups && meta->compile_subgroups[device_i]))
+    flags |= POCL_KERNEL_HAS_SUBG_META;
+  if ((meta->local_mem_size && meta->local_mem_size[device_i])
+      || (meta->private_mem_size && meta->private_mem_size[device_i])
+      || (meta->spill_mem_size && meta->spill_mem_size[device_i])
+      || (meta->max_workgroup_size && meta->max_workgroup_size[device_i])
+      || (meta->preferred_wg_multiple
+          && meta->preferred_wg_multiple[device_i]))
+    flags |= POCL_KERNEL_HAS_WORKG_META;
+  BUFFER_STORE (flags, uint32_t);
+
+  if (flags & POCL_KERNEL_HAS_SUBG_META)
+    {
+      uint32_t tmp = 0;
+      if (meta->max_subgroups)
+        tmp = meta->max_subgroups[device_i];
+      BUFFER_STORE (tmp, uint32_t);
+      tmp = 0;
+      if (meta->compile_subgroups)
+        tmp = meta->compile_subgroups[device_i];
+      BUFFER_STORE (tmp, uint32_t);
+    }
+
+  if (flags & POCL_KERNEL_HAS_WORKG_META)
+    {
+      uint32_t tmp;
+      tmp = 0;
+      if (meta->max_workgroup_size)
+        tmp = meta->max_workgroup_size[device_i];
+      BUFFER_STORE (tmp, uint32_t);
+      tmp = 0;
+      if (meta->preferred_wg_multiple)
+        tmp = meta->preferred_wg_multiple[device_i];
+      BUFFER_STORE (tmp, uint32_t);
+      tmp = 0;
+      if (meta->local_mem_size)
+        tmp = meta->local_mem_size[device_i];
+      BUFFER_STORE (tmp, uint32_t);
+      tmp = 0;
+      if (meta->private_mem_size)
+        tmp = meta->private_mem_size[device_i];
+      BUFFER_STORE (tmp, uint32_t);
+      tmp = 0;
+      if (meta->spill_mem_size)
+        tmp = meta->spill_mem_size[device_i];
+      BUFFER_STORE (tmp, uint32_t);
+    }
 
   /***********************************************************************/
   unsigned char *start = buffer;
@@ -407,8 +488,19 @@ pocl_binary_serialize_kernel_to_buffer(cl_program program,
       BUFFER_STORE(ai->type_qualifier, cl_kernel_arg_type_qualifier);
       BUFFER_STORE(ai->type, uint32_t);
       BUFFER_STORE (ai->type_size, uint32_t);
-      BUFFER_STORE_STR(ai->name);
-      BUFFER_STORE_STR(ai->type_name);
+      if (meta->has_arg_metadata & POCL_HAS_KERNEL_ARG_NAME)
+        BUFFER_STORE_STR(ai->name);
+      else
+      {
+        char temp[2];
+        temp[0] = 'a' + i;
+        temp[1] = 0;
+        BUFFER_STORE_STR(temp);
+      }
+      if (ai->type_name)
+        BUFFER_STORE_STR(ai->type_name);
+      else
+        BUFFER_STORE(0, uint32_t);
     }
   /***********************************************************************/
 
@@ -447,7 +539,6 @@ deserialize_file (unsigned char* buffer,
 
   char* content = NULL;
   BUFFER_READ_STR2 (content, len);
-  assert (len > 0);
 
   char *p = basedir + offset;
   strcpy (p, relpath);
@@ -463,7 +554,10 @@ deserialize_file (unsigned char* buffer,
     pocl_mkdir_p (dirpath);
   free (dir);
 
-  pocl_write_file (fullpath, content, len, 0, 0);
+  if (len == 0)
+    pocl_touch_file (fullpath);
+  else
+    pocl_write_file (fullpath, content, len, 0, 0);
 
 RET:
   free (content);
@@ -472,7 +566,7 @@ RET:
 
 /* Deserializes all files of a single pocl kernel cachedir.  */
 static unsigned char*
-deserialize_kernel_cachedir (char* basedir, unsigned char* buffer, size_t bytes)
+recursively_deserialize_path (char* basedir, unsigned char* buffer, size_t bytes)
 {
   size_t done = 0;
   size_t offset = strlen (basedir);
@@ -481,6 +575,7 @@ deserialize_kernel_cachedir (char* basedir, unsigned char* buffer, size_t bytes)
     {
       done += deserialize_file (buffer + done, basedir, offset);
     }
+  basedir[offset] = 0;
   assert(done == bytes);
   return (buffer + done);
 }
@@ -509,7 +604,6 @@ pocl_binary_deserialize_kernel_from_buffer (pocl_binary *b,
 {
   unsigned i;
   unsigned char *buffer = *buf;
-  uint64_t *dynarg_sizes;
 
   memset(kernel, 0, sizeof(pocl_binary_kernel));
   BUFFER_READ(kernel->struct_size, uint64_t);
@@ -519,23 +613,12 @@ pocl_binary_deserialize_kernel_from_buffer (pocl_binary *b,
   BUFFER_READ(kernel->num_args, uint32_t);
   BUFFER_READ(kernel->num_locals, uint32_t);
 
-  dynarg_sizes = alloca (sizeof(uint64_t) * kernel->num_args);
-
-  for (i = 0; i < OPENCL_MAX_DIMENSION; i++)
-    {
-      BUFFER_READ(kernel->reqd_wg_size[i], uint64_t);
-    }
-
   if (meta)
     {
-      if (b->version < 7)
+      for (i = 0; i < OPENCL_MAX_DIMENSION; i++)
         {
-          for (i = 0; i < kernel->num_args; i++)
-            {
-              BUFFER_READ (dynarg_sizes[i], uint64_t);
-            }
+          BUFFER_READ (kernel->reqd_wg_size[i], uint64_t);
         }
-
       kernel->local_sizes = calloc (kernel->num_locals, sizeof (size_t));
       for (i = 0; i < kernel->num_locals; i++)
         {
@@ -544,15 +627,23 @@ pocl_binary_deserialize_kernel_from_buffer (pocl_binary *b,
           kernel->local_sizes[i] = temp;
         }
 
-      if (b->version >= 7)
+      BUFFER_READ_STR2(kernel->attributes, kernel->sizeof_attributes);
+      BUFFER_READ (kernel->has_arg_metadata, uint32_t);
+      BUFFER_READ (kernel->flags, uint32_t);
+
+      if (kernel->flags & POCL_KERNEL_HAS_SUBG_META)
         {
-          BUFFER_READ_STR2(kernel->attributes, kernel->sizeof_attributes);
-          BUFFER_READ(kernel->has_arg_metadata, uint64_t);
+          BUFFER_READ (kernel->max_subgroups, uint32_t);
+          BUFFER_READ (kernel->compile_subgroups, uint32_t);
         }
-      else
+
+      if (kernel->flags & POCL_KERNEL_HAS_WORKG_META)
         {
-          kernel->attributes = NULL;
-          kernel->has_arg_metadata = (-1);
+          BUFFER_READ (kernel->max_wg_size, uint32_t);
+          BUFFER_READ (kernel->preferred_wg_multiple, uint32_t);
+          BUFFER_READ (kernel->local_mem_size, uint32_t);
+          BUFFER_READ (kernel->private_mem_size, uint32_t);
+          BUFFER_READ (kernel->spill_mem_size, uint32_t);
         }
 
       meta->arg_info = calloc (kernel->num_args, sizeof (struct pocl_argument_info));
@@ -564,22 +655,9 @@ pocl_binary_deserialize_kernel_from_buffer (pocl_binary *b,
           BUFFER_READ (ai->access_qualifier, cl_kernel_arg_access_qualifier);
           BUFFER_READ (ai->address_qualifier, cl_kernel_arg_address_qualifier);
           BUFFER_READ (ai->type_qualifier, cl_kernel_arg_type_qualifier);
-          if (b->version < 7)
-            {
-              char t1, t2;
-              BUFFER_READ (t1, char);
-              BUFFER_READ (t2, char);
-            }
 
           BUFFER_READ (ai->type, uint32_t);
-          if (b->version >= 7)
-            {
-              BUFFER_READ (ai->type_size, uint32_t);
-            }
-          else
-            {
-              ai->type_size = dynarg_sizes[i];
-            }
+          BUFFER_READ (ai->type_size, uint32_t);
           BUFFER_READ_STR (ai->name);
           BUFFER_READ_STR (ai->type_name);
         }
@@ -589,7 +667,8 @@ pocl_binary_deserialize_kernel_from_buffer (pocl_binary *b,
     {
       /* skip the arg_info and all kernel metadata */
       buffer = *buf + (kernel->struct_size - kernel->binaries_size);
-      deserialize_kernel_cachedir (basedir, buffer, kernel->binaries_size);
+      if (kernel->binaries_size > 0)
+        recursively_deserialize_path (basedir, buffer, kernel->binaries_size);
       POCL_MEM_FREE (kernel->kernel_name);
     }
 
@@ -601,9 +680,15 @@ pocl_binary_deserialize_kernel_from_buffer (pocl_binary *b,
 
 /***********************************************************/
 
+static const char* DEFAULT_ENTRIES[] = { "/program.bc", "/" POCL_GVAR_INIT_KERNEL_NAME };
+static const unsigned NUM_DEFAULT_ENTRIES = 2;
+
 cl_int
 pocl_binary_serialize(cl_program program, unsigned device_i, size_t *size)
 {
+  unsigned i;
+
+  cl_device_id dev = program->devices[device_i];
   unsigned char *buffer = program->pocl_binaries[device_i];
   size_t sizeof_buffer = program->pocl_binary_sizes[device_i];
   unsigned char *end_of_buffer = buffer + sizeof_buffer;
@@ -611,30 +696,59 @@ pocl_binary_serialize(cl_program program, unsigned device_i, size_t *size)
 
   unsigned num_kernels = program->num_kernels;
 
+  char basedir[POCL_MAX_PATHNAME_LENGTH];
+  pocl_cache_program_path (basedir, program, device_i);
+  size_t basedir_len = strlen (basedir);
+
   memcpy(buffer, POCLCC_STRING_ID, POCLCC_STRING_ID_LENGTH);
   buffer += POCLCC_STRING_ID_LENGTH;
   BUFFER_STORE(pocl_binary_get_device_id(program->devices[device_i]), uint64_t);
   BUFFER_STORE(POCLCC_VERSION, uint32_t);
   BUFFER_STORE(num_kernels, uint32_t);
-  uint64_t flags = 0;
+  uint64_t flags = POCL_BINARY_HAS_PROG_SCOPE_VARS;
   if (program->flush_denorms)
     flags |= POCL_BINARY_FLAG_FLUSH_DENORMS;
-  flags |= (program->binary_type << 1);
+  flags |= ((uint64_t)program->binary_type << 32);
   BUFFER_STORE (flags, uint64_t);
+
+  unsigned char *root_entries_save = buffer;
+  // to be replaced with actual number of saved entries later
+  BUFFER_STORE (0, uint32_t);
+
   memcpy(buffer, program->build_hash[device_i], sizeof(SHA1_digest_t));
   buffer += sizeof(SHA1_digest_t);
-
+  BUFFER_STORE (program->global_var_total_size[device_i], uint64_t);
   assert(buffer < end_of_buffer);
 
-  char basedir[POCL_FILENAME_LENGTH];
-  pocl_cache_program_path (basedir, program, device_i);
-  size_t basedir_len = strlen (basedir);
-  char program_bc_path[POCL_FILENAME_LENGTH];
-  pocl_cache_program_bc_path (program_bc_path, program, device_i);
-  POCL_MSG_PRINT_INFO ("serializing program.bc: %s\n", program_bc_path);
-  buffer = serialize_file (program_bc_path, basedir_len, buffer);
+  unsigned actually_serialized_entries = 0;
+  if (dev->num_serialize_entries == 0)
+  {
+    dev->num_serialize_entries = NUM_DEFAULT_ENTRIES;
+    dev->serialize_entries = DEFAULT_ENTRIES;
+  }
+  for (i = 0; i < dev->num_serialize_entries; ++i)
+  {
+    unsigned char *saved_b = buffer;
+    BUFFER_STORE(0, uint64_t); // size of the following binaries;
+    char temp[POCL_MAX_PATHNAME_LENGTH];
+    strcpy(temp, basedir);
+    strcat(temp, dev->serialize_entries[i]);
+    POCL_MSG_PRINT_INFO ("serializing %s\n", temp);
+    unsigned char* new_buffer = recursively_serialize_path (temp, basedir_len, buffer);
+    uint64_t size = new_buffer - buffer;
+    buffer = saved_b;
+    if (size > 0) {
+      BUFFER_STORE (size, uint64_t);
+      buffer = new_buffer;
+      ++actually_serialized_entries;
+    }
+  }
+  unsigned char* temp = buffer;
+  buffer = root_entries_save;
+  BUFFER_STORE (actually_serialized_entries, uint32_t);
+  buffer = temp;
+
 
-  unsigned i;
   for (i=0; i < num_kernels; i++)
     {
       buffer = pocl_binary_serialize_kernel_to_buffer
@@ -650,6 +764,9 @@ pocl_binary_serialize(cl_program program, unsigned device_i, size_t *size)
 cl_int
 pocl_binary_deserialize(cl_program program, unsigned device_i)
 {
+  unsigned i;
+
+  cl_device_id dev = program->devices[device_i];
   unsigned char *buffer = program->pocl_binaries[device_i];
   size_t sizeof_buffer = program->pocl_binary_sizes[device_i];
   unsigned char *end_of_buffer = buffer + sizeof_buffer;
@@ -657,21 +774,37 @@ pocl_binary_deserialize(cl_program program, unsigned device_i)
   pocl_binary b;
   buffer = read_header(&b, buffer);
   program->flush_denorms = (b.flags & POCL_BINARY_FLAG_FLUSH_DENORMS);
-  program->binary_type = (b.flags >> 1);
+  program->binary_type = (b.flags >> 32);
+  program->global_var_total_size[device_i] = b.program_scope_var_bytes;
+
+  if (dev->num_serialize_entries == 0)
+  {
+    dev->num_serialize_entries = NUM_DEFAULT_ENTRIES;
+    dev->serialize_entries = DEFAULT_ENTRIES;
+  }
+
+  assert (b.root_entries <= dev->num_serialize_entries);
 
   //assert(pocl_binary_check_binary_header(&b));
   assert (buffer < end_of_buffer);
 
-  char basedir[POCL_FILENAME_LENGTH];
+  char basedir[POCL_MAX_PATHNAME_LENGTH];
   pocl_cache_program_path (basedir, program, device_i);
   size_t basedir_len = strlen (basedir);
-  buffer += deserialize_file (buffer, basedir, basedir_len);
+
+  for (i = 0; i < b.root_entries; ++i)
+  {
+    uint64_t bytes;
+    BUFFER_READ(bytes, uint64_t);
+    unsigned char* retval = recursively_deserialize_path (basedir, buffer, bytes);
+    assert (retval == buffer + bytes);
+    buffer += bytes;
+  }
 
   pocl_binary_kernel k;
-  unsigned i;
   for (i = 0; i < b.num_kernels; i++)
     {
-      pocl_cache_program_path (basedir, program, device_i);
+      basedir[basedir_len] = 0;
       if (pocl_binary_deserialize_kernel_from_buffer (&b, &buffer, &k, NULL, basedir)
           != CL_SUCCESS)
         goto ERROR;
@@ -723,6 +856,7 @@ pocl_binary_get_kernels_metadata (cl_program program, unsigned device_i)
 {
   unsigned char *binary = program->pocl_binaries[device_i];
   cl_device_id device = program->devices[device_i];
+  size_t max_len = program->pocl_binary_sizes[device_i];
 
   pocl_binary b;
   memset(&b, 0, sizeof (pocl_binary));
@@ -730,20 +864,20 @@ pocl_binary_get_kernels_metadata (cl_program program, unsigned device_i)
   memset(&k, 0, sizeof (pocl_binary_kernel));
 
   unsigned char* buffer = read_header (&b, binary);
+  unsigned char* start = buffer;
   POCL_RETURN_ERROR_ON ((!pocl_binary_check_binary (device, binary)),
                         CL_INVALID_PROGRAM,
                         "Deserialized a binary, but it doesn't seem to be "
                         "for this device.\n");
-  size_t len;
-  /* skip real path of program.bc */
-  BUFFER_READ(len, uint32_t);
-  assert (len > 0);
-  buffer += len;
-
-  /* skip content of program.bc */
-  BUFFER_READ(len, uint32_t);
-  assert (len > 0);
-  buffer += len;
+  unsigned i;
+  for (i = 0; i < b.root_entries; ++i)
+    {
+      uint64_t len;
+      BUFFER_READ(len, uint64_t);
+      assert (len > 0);
+      buffer += len;
+      assert (buffer - start <= max_len);
+    }
 
   unsigned j;
   assert (b.num_kernels > 0);
@@ -769,6 +903,47 @@ pocl_binary_get_kernels_metadata (cl_program program, unsigned device_i)
           = (void **)calloc (program->associated_num_devices, sizeof (void *));
       assert (km->name);
 
+      if (k.flags & POCL_KERNEL_HAS_SUBG_META)
+        {
+          if (km->max_subgroups == NULL)
+            km->max_subgroups = (size_t *)calloc (
+                program->associated_num_devices, sizeof (size_t));
+          km->max_subgroups[device_i] = k.max_subgroups;
+
+          if (km->compile_subgroups == NULL)
+            km->compile_subgroups = (size_t *)calloc (
+                program->associated_num_devices, sizeof (size_t));
+          km->compile_subgroups[device_i] = k.compile_subgroups;
+        }
+
+      if (k.flags & POCL_KERNEL_HAS_WORKG_META)
+        {
+          if (km->max_workgroup_size == NULL)
+            km->max_workgroup_size = (size_t *)calloc (
+                program->associated_num_devices, sizeof (size_t));
+          km->max_workgroup_size[device_i] = k.max_wg_size;
+
+          if (km->preferred_wg_multiple == NULL)
+            km->preferred_wg_multiple = (size_t *)calloc (
+                program->associated_num_devices, sizeof (size_t));
+          km->preferred_wg_multiple[device_i] = k.preferred_wg_multiple;
+
+          if (km->local_mem_size == NULL)
+            km->local_mem_size = (cl_ulong *)calloc (
+                program->associated_num_devices, sizeof (cl_ulong));
+          km->local_mem_size[device_i] = k.local_mem_size;
+
+          if (km->private_mem_size == NULL)
+            km->private_mem_size = (cl_ulong *)calloc (
+                program->associated_num_devices, sizeof (cl_ulong));
+          km->private_mem_size[device_i] = k.private_mem_size;
+
+          if (km->spill_mem_size == NULL)
+            km->spill_mem_size = (cl_ulong *)calloc (
+                program->associated_num_devices, sizeof (cl_ulong));
+          km->spill_mem_size[device_i] = k.spill_mem_size;
+        }
+
       unsigned l;
       for (l = 0; l < OPENCL_MAX_DIMENSION; l++)
         {
