@@ -1,7 +1,7 @@
 /* OpenCL runtime library: compile_and_link_program()
 
    Copyright (c) 2011-2013 Universidad Rey Juan Carlos,
-                 2011-2019 Pekka Jääskeläinen
+                 2011-2023 Pekka Jääskeläinen
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
@@ -91,8 +91,15 @@ static const char cl_program_link_options[] =
   "-cl-finite-math-only "
   "-cl-fast-relaxed-math ";
 
-static const char cl_parameters_not_yet_supported_by_clang[] =
-  "-cl-uniform-work-group-size ";
+/* TODO: In case of a PoCL-R, we should pass on unhandled
+   target-specific/extension-specific options to the
+   native driver's clBuildProgram() call. Might be difficult
+   to filter the kept ones as the set of target devices can
+   be diverse. */
+static const char cl_parameters_not_yet_supported_by_clang[]
+    = "-cl-uniform-work-group-size "
+      "-cl-no-subgroup-ifp "
+      "-cl-intel-no-prera-scheduling";
 
 #define MEM_ASSERT(x, err_jmp) do{ if (x){errcode = CL_OUT_OF_HOST_MEMORY;goto err_jmp;}} while(0)
 
@@ -519,8 +526,6 @@ setup_kernel_metadata (cl_program program)
         }
       else
         {
-          assert (program->source || program->binaries[device_i]
-                  || (program->num_builtin_kernels > 0));
           if (device->ops->setup_metadata
               && device->ops->setup_metadata (device, program, device_i))
             {
@@ -704,6 +709,12 @@ compile_and_link_program(int compile_program,
         goto ERROR_CLEAN_OPTIONS;
     }
 
+  /* The build option -x spir is only needed for the old SPIR format.
+     When creating a SPIR-V program via clCreateProgramWithIL, it's not
+     needed and we just assume if the program_il blob is there, we want
+     to also build it. */
+  spir_build = spir_build || program->program_il != NULL;
+
   POCL_MSG_PRINT_LLVM ("building program with options %s\n",
                        program->compiler_options);
 
@@ -753,6 +764,8 @@ compile_and_link_program(int compile_program,
   for (i = 0; i < program->num_devices; ++i)
     {
       cl_device_id dev = program->devices[i];
+      POCL_GOTO_ERROR_COND ((*dev->available == CL_FALSE),
+                            CL_DEVICE_NOT_AVAILABLE);
       for (cl_uint j = 0; j < program->associated_num_devices; ++j)
         if (program->associated_devices[j] == dev)
           ++num_found;
@@ -876,7 +889,7 @@ compile_and_link_program(int compile_program,
         }
 
       /* Maintain a 'last_accessed' file in every program's
-       * cache directory. Will be useful for cache pruning script
+       * cache directory. Will be useful for a cache pruning script
        * that flushes old directories based on LRU */
       if (!program->builtin_kernel_names)
         pocl_cache_update_program_last_access (program, device_i);
@@ -908,7 +921,14 @@ compile_and_link_program(int compile_program,
       cl_device_id device = program->devices[device_i];
 
       if (device->ops->post_build_program)
-        device->ops->post_build_program (program, device_i);
+        {
+          errcode = device->ops->post_build_program (program, device_i);
+          if (errcode != CL_SUCCESS)
+            {
+              POCL_MSG_ERR ("Program build: post-build-program failed\n");
+              goto ERROR;
+            }
+        }
     }
 
   TP_BUILD_PROGRAM (program->context->id, program->id);
