@@ -189,6 +189,10 @@ POdeclsym(clEnqueueMemcpyINTEL)
 POdeclsym(clEnqueueMemAdviseINTEL)
 POdeclsym(clEnqueueMigrateMemINTEL)
 
+/* cl_khr_command_buffer 0.9.4 */
+POdeclsym(clCommandSVMMemcpyKHR)
+POdeclsym(clCommandSVMMemFillKHR)
+
 
 #ifdef __cplusplus
 }
