@@ -50,8 +50,9 @@ IGNORE_COMPILER_WARNING("-Wstrict-aliasing")
 #include <llvm/IR/Module.h>
 #include <llvm/IR/Verifier.h>
 
-
-#ifndef LLVM_OLDER_THAN_11_0
+#if LLVM_VERSION_MAJOR > 15
+#include "llvm/TargetParser/Host.h"
+#elif LLVM_VERSION_MAJOR > 10
 #include "llvm/Support/Host.h"
 #endif
 
@@ -179,7 +180,7 @@ static bool generateProgramBC(PoclLLVMContextData *Context, llvm::Module *Mod,
     return true;
 
 #ifndef LLVM_OLDER_THAN_14_0
-  if (Device->program_scope_variables_pass) {
+  if (Device->run_program_scope_variables_pass) {
     size_t TotalGVarBytes = 0;
     if (runProgramScopeVariablesPass(Mod, Device->global_as_id,
                                      Device->local_as_id, TotalGVarBytes, Log))
@@ -188,7 +189,8 @@ static bool generateProgramBC(PoclLLVMContextData *Context, llvm::Module *Mod,
   }
 #endif
 
-  if (link(Mod, BuiltinLib, Log, Device->device_aux_functions))
+  if (link(Mod, BuiltinLib, Log, Device->device_aux_functions,
+           Device->device_side_printf != CL_FALSE))
     return true;
 
   raw_string_ostream OS(Log);
@@ -410,6 +412,8 @@ int pocl_llvm_build_program(cl_program program,
   ss << "-triple=" << device->llvm_target_triplet << " ";
   if (device->llvm_cpu != NULL)
     ss << "-target-cpu " << device->llvm_cpu << " ";
+  if (device->llvm_abi != NULL)
+    ss << "-target-abi " << device->llvm_abi << " ";
 
   std::string AllBuildOpts = ss.str();
 
@@ -518,6 +522,7 @@ int pocl_llvm_build_program(cl_program program,
   la->Blocks = true; //-fblocks
   la->MathErrno = false; // -fno-math-errno
   la->NoBuiltin = true;  // -fno-builtin
+  la->Freestanding = true; // -ffree-standing
   la->AsmBlocks = true;  // -fasm (?)
 
   // setLangDefaults overrides to FPM_On for OpenCL.
@@ -942,46 +947,6 @@ int pocl_llvm_link_program(cl_program program, unsigned device_i,
   return CL_SUCCESS;
 }
 
-/* for "distro" style kernel libs, return which kernellib to use, at runtime */
-#ifdef KERNELLIB_HOST_DISTRO_VARIANTS
-const char *getX86KernelLibName() {
-  StringMap<bool> Features;
-  const char *res = NULL;
-
-  if (!llvm::sys::getHostCPUFeatures(Features)) {
-    POCL_MSG_WARN ("getX86KernelLibName(): LLVM can't get host CPU flags!\n");
-    /* getX86KernelLibName should only ever be enabled
-       on x86-64, which always has sse2 */
-    return "sse2";
-  }
-
-  if (Features["sse2"])
-    res = "sse2";
-  else
-    POCL_ABORT("Pocl on x86_64 requires at least SSE2\n");
-  if (Features["ssse3"] && Features["cx16"])
-    res = "ssse3";
-  if (Features["sse4.1"] && Features["cx16"])
-    res = "sse41";
-  if (Features["avx"] && Features["cx16"] && Features["popcnt"])
-    res = "avx";
-  if (Features["avx"] && Features["cx16"] && Features["popcnt"] && Features["f16c"])
-    res = "avx_f16c";
-  if (Features["avx"] && Features["cx16"] && Features["popcnt"]
-      && Features["xop"] && Features["fma4"])
-    res = "avx_fma4";
-  if (Features["avx"] && Features["avx2"] && Features["cx16"]
-      && Features["popcnt"] && Features["lzcnt"] && Features["f16c"]
-      && Features["fma"] && Features["bmi"] && Features["bmi2"])
-    res = "avx2";
-  if (Features["avx512f"] )
-    res = "avx512";
-
-  return res;
-}
-#endif
-
-
 /**
  * Return the OpenCL C built-in function library bitcode
  * for the given device.
@@ -995,71 +960,31 @@ static llvm::Module *getKernelLibrary(cl_device_id device,
   if (kernelLibraryMap->find(device) != kernelLibraryMap->end())
     return kernelLibraryMap->at(device);
 
-  const char *subdir = "host";
-  bool is_host = true;
-  // TODO: move this to the device layer, a property to ask for
-  // the kernel builtin bitcode library name, including its subdir
-#if defined(TCE_AVAILABLE)
-  if (triple.getArch() == Triple::tce || triple.getArch() == Triple::tcele) {
-    subdir = "tce";
-    is_host = false;
-  }
-#endif
-#ifdef BUILD_HSA
-  if (triple.getArch() == Triple::hsail64) {
-    subdir = "hsail64";
-    is_host = false;
-  }
-#endif
-#ifdef AMDGCN_ENABLED
-  if (triple.getArch == Triple::amdgcn) {
-    subdir = "amdgcn";
-    is_host = false;
-  }
-#endif
-#ifdef BUILD_CUDA
-  if (triple.getArch() == Triple::nvptx ||
-      triple.getArch() == Triple::nvptx64) {
-    subdir = "cuda";
-    is_host = false;
-  }
-#endif
-
-  std::string kernellib;
-  std::string kernellib_fallback;
+  std::string kernellib_common, kernellib, kernellib_fallback;
 
 #ifdef ENABLE_POCL_BUILDING
   if (pocl_get_bool_option("POCL_BUILDING", 0)) {
-    kernellib = BUILDDIR;
-    kernellib += "/lib/kernel/";
-    kernellib += subdir;
+    kernellib_common = BUILDDIR;
+    kernellib_common += "/lib/kernel/";
+    kernellib_common += device->kernellib_subdir;
   } else // POCL_BUILDING == 0, use install dir
 #endif
   {
     char temp[POCL_MAX_PATHNAME_LENGTH];
     pocl_get_private_datadir(temp);
-    kernellib = temp;
+    kernellib_common = temp;
   }
 
-  kernellib += "/kernel-";
-  kernellib += device->llvm_target_triplet;
+  kernellib_common += "/";
 
-  if (is_host) {
-    kernellib += '-';
-#ifdef KERNELLIB_HOST_DISTRO_VARIANTS
-    kernellib += getX86KernelLibName();
-#elif defined(HOST_CPU_FORCED)
-    kernellib += OCL_KERNEL_TARGET_CPU;
-#else
-    kernellib_fallback = kernellib;
-    kernellib_fallback += OCL_KERNEL_TARGET_CPU;
-    kernellib_fallback += ".bc";
-    if (device->llvm_cpu)
-      kernellib += device->llvm_cpu;
-#endif
-  }
+  kernellib = kernellib_common + device->kernellib_name;
   kernellib += ".bc";
 
+  if (device->kernellib_fallback_name) {
+    kernellib_fallback = kernellib_common + device->kernellib_fallback_name;
+    kernellib_fallback += ".bc";
+  }
+
   llvm::Module *lib;
 
   if (pocl_exists(kernellib.c_str()))
@@ -1069,15 +994,13 @@ static llvm::Module *getKernelLibrary(cl_device_id device,
     }
   else
     {
-#if !defined(KERNELLIB_HOST_DISTRO_VARIANTS) && !defined(HOST_CPU_FORCED)
-      if (is_host && pocl_exists(kernellib_fallback.c_str()))
+      if (device->kernellib_fallback_name && pocl_exists(kernellib_fallback.c_str()))
         {
           POCL_MSG_WARN("Using fallback %s as the built-in lib.\n",
                         kernellib_fallback.c_str());
           lib = parseModuleIR(kernellib_fallback.c_str(), llvmContext);
         }
       else
-#endif
         POCL_ABORT("Kernel library file %s doesn't exist.\n", kernellib.c_str());
     }
   assert (lib != NULL);
