@@ -22,48 +22,57 @@
    THE SOFTWARE.
 */
 
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wmaybe-uninitialized")
+#include <llvm/ADT/StringRef.h>
+#include <llvm/ADT/StringMap.h>
+#if LLVM_VERSION_MAJOR < 16
+#include <llvm/ADT/Triple.h>
+#else
+#include <llvm/TargetParser/Triple.h>
+#endif
+POP_COMPILER_DIAGS
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
+#include <llvm/Support/TargetSelect.h>
+#include <llvm/Support/SourceMgr.h>
+#include <llvm/Support/Signals.h>
+#include <llvm/IR/Constants.h>
+#include <llvm/IR/DiagnosticInfo.h>
+#include <llvm/IR/DiagnosticPrinter.h>
+#include <llvm/IR/LLVMContext.h>
+#include <llvm/IR/Module.h>
+#include <llvm/Target/TargetMachine.h>
+#include <llvm/IRReader/IRReader.h>
+#include <llvm/Bitcode/BitcodeReader.h>
+#include <llvm/Bitcode/BitcodeWriter.h>
+#include <llvm/Support/raw_os_ostream.h>
+#include <llvm/PassRegistry.h>
+#include <llvm/InitializePasses.h>
+#include <llvm/Support/CommandLine.h>
+#include <llvm-c/Core.h>
+
+#if LLVM_VERSION_MAJOR < 16
+#include <llvm/IR/LegacyPassManager.h>
+#include <llvm/PassInfo.h>
+#include <llvm/PassRegistry.h>
+#include <llvm/Support/Host.h>
+#else
+#include <llvm/Analysis/CGSCCPassManager.h>
+#include <llvm/Analysis/LoopAnalysisManager.h>
+#include <llvm/Passes/PassBuilder.h>
+#include <llvm/TargetParser/Host.h>
+#endif
+
 #include "config.h"
 #include "pocl_debug.h"
 #include "pocl_file_util.h"
 #include "pocl_llvm.h"
 #include "pocl_llvm_api.h"
 #include "pocl_runtime_config.h"
+
 #include <unistd.h>
 
-#include "CompilerWarnings.h"
-IGNORE_COMPILER_WARNING("-Wunused-parameter")
-
-#include <llvm/ADT/StringRef.h>
-#include <llvm/ADT/StringMap.h>
-
-#include <llvm/Support/Host.h>
-#include <llvm/Support/TargetSelect.h>
-#include <llvm/Support/SourceMgr.h>
-#include <llvm/Support/Signals.h>
-
-#include <llvm/IR/Constants.h>
-#include <llvm/IR/DiagnosticInfo.h>
-#include <llvm/IR/DiagnosticPrinter.h>
-#include <llvm/IR/LLVMContext.h>
-#include <llvm/IR/Module.h>
-
-#include <llvm/Target/TargetMachine.h>
-
-#include <llvm/IRReader/IRReader.h>
-
-#include <llvm/Bitcode/BitcodeReader.h>
-#include <llvm/Bitcode/BitcodeWriter.h>
-
-#include <llvm/Support/raw_os_ostream.h>
-#include <llvm/PassRegistry.h>
-
-#include <llvm/IR/LegacyPassManager.h>
-#define PassManager legacy::PassManager
-
-#include <llvm/InitializePasses.h>
-#include <llvm/Support/CommandLine.h>
-#include <llvm-c/Core.h>
-
 using namespace llvm;
 
 #include <string>
@@ -135,8 +144,38 @@ static int getModuleTriple(const char *input_stream, size_t size,
   return 0;
 }
 
+const char *pocl_get_llvm_cpu_abi() {
+#ifdef HOST_CPU_TARGET_ABI
+  if (strlen(HOST_CPU_TARGET_ABI) > 0)
+    return HOST_CPU_TARGET_ABI;
+  else {
+#if defined(__riscv) && (HOST_DEVICE_ADDRESS_BITS == 64)
+
+#ifdef __riscv_float_abi_soft
+    return "lp64";
+#endif
+
+#ifdef __riscv_float_abi_single
+    return "lp64f";
+#endif
+
+#ifdef __riscv_float_abi_double
+    return "lp64d";
+#endif
+
+#ifdef __riscv_float_abi_quad
+    return "lp64q";
+#endif
+
+#endif // __riscv
+  }
+#endif // HOST_CPU_TARGET_ABI
+  return nullptr;
+}
+
 char *pocl_get_llvm_cpu_name() {
-  StringRef r = llvm::sys::getHostCPUName();
+  const char *custom = pocl_get_string_option("POCL_LLVM_CPU_NAME", NULL);
+  StringRef r = custom ? StringRef(custom) : llvm::sys::getHostCPUName();
 
   // LLVM may return an empty string -- treat as generic
   if (r.empty())
@@ -146,7 +185,7 @@ char *pocl_get_llvm_cpu_name() {
   if (r.str() == "generic" && strlen(OCL_KERNEL_TARGET_CPU)) {
     POCL_MSG_WARN("LLVM does not recognize your cpu, trying to use "
                    OCL_KERNEL_TARGET_CPU " for -target-cpu\n");
-    r = llvm::StringRef(OCL_KERNEL_TARGET_CPU);
+    r = StringRef(OCL_KERNEL_TARGET_CPU);
   }
 #endif
 
@@ -157,7 +196,125 @@ char *pocl_get_llvm_cpu_name() {
   return cpu_name;
 }
 
-int bitcode_is_triple(const char *bitcode, size_t size, const char *triple) {
+#ifdef KERNELLIB_HOST_DISTRO_VARIANTS
+const struct kernellib_features {
+  const char *kernellib_variant;
+  const char *cpu_name;
+  const char *features[12];
+} kernellib_feature_map[] = {
+// order the entries s.t. if a cpu matches multiple entries, the "best" match
+// comes last
+#if defined(__i386__)
+    "i386",
+    "i386",
+    {NULL},
+    // FIXME: Can we distinguish i686 from i386 using feature flags?
+    "i686",
+    "i686",
+    {NULL},
+    "mmx",
+    "pentium-mmx",
+    {"mmx", NULL},
+    "sse",
+    "pentium3",
+    {"sse", NULL},
+#endif
+#if defined(__i386__) || defined(__x86_64__)
+    "sse2",
+    "x86-64",
+    {"sse2", NULL},
+    "ssse3",
+    "core2",
+    {"sse2", "ssse3", "cx16", NULL},
+    "sse41",
+    "penryn",
+    {"sse2", "sse4.1", "cx16", NULL},
+    "avx",
+    "sandybridge",
+    {"sse2", "avx", "cx16", "popcnt", NULL},
+    "avx_f16c",
+    "ivybridge",
+    {"sse2", "avx", "cx16", "popcnt", "f16c", NULL},
+    "avx_fma4",
+    "bdver1",
+    {"sse2", "avx", "cx16", "popcnt", "xop", "fma4", NULL},
+    "avx2",
+    "haswell",
+    {"sse2", "avx", "avx2", "cx16", "popcnt", "lzcnt", "f16c", "fma", "bmi",
+     "bmi2", NULL},
+    "avx512",
+    "skylake-avx512",
+    {"sse2", "avx512f", NULL},
+#endif
+    NULL,
+    NULL,
+    {NULL}};
+
+/* for "distro" style kernel libs, return which kernellib to use, at runtime */
+const char *pocl_get_distro_kernellib_variant() {
+  StringMap<bool> Features;
+
+#if defined(__x86_64__)
+  if (!llvm::sys::getHostCPUFeatures(Features)) {
+    POCL_MSG_WARN("LLVM can't get host CPU flags!\n");
+    return NULL;
+  }
+#else
+  return pocl_get_llvm_cpu_name();
+#endif
+
+  const char *custom = pocl_get_string_option("POCL_KERNELLIB_NAME", NULL);
+
+  const kernellib_features *best_match = NULL;
+  for (const kernellib_features *kf = kernellib_feature_map; kf->kernellib_variant;
+       ++kf) {
+    bool matches = true;
+    for (const char *const *f = kf->features; *f; ++f)
+      matches &= Features[*f];
+    if (matches) {
+      best_match = kf;
+      if (custom && !strcmp(custom, kf->kernellib_variant))
+        break;
+    }
+  }
+
+  if (!best_match) {
+    POCL_MSG_WARN("Can't find a kernellib supported by the host CPU (%s)\n",
+                  llvm::sys::getHostCPUName());
+    return NULL;
+  }
+
+  return best_match->kernellib_variant;
+}
+
+/* for "distro" style kernel libs, return which target cpu to use for a given
+ * kernellib */
+const char *pocl_get_distro_cpu_name(const char *kernellib_variant) {
+  StringMap<bool> Features;
+
+#if defined(__x86_64__)
+  if (!llvm::sys::getHostCPUFeatures(Features)) {
+    POCL_MSG_WARN("LLVM can't get host CPU flags!\n");
+    return NULL;
+  }
+#else
+  return pocl_get_llvm_cpu_name();
+#endif
+
+  const kernellib_features *best_match = NULL;
+  for (const kernellib_features *kf = kernellib_feature_map; kf->kernellib_variant;
+       ++kf) {
+    if (!strcmp(kernellib_variant, kf->kernellib_variant))
+      return kf->cpu_name;
+  }
+
+  POCL_MSG_WARN("Can't find a cpu name matching the kernellib (%s)\n",
+                kernellib_variant);
+  return NULL;
+}
+#endif
+
+int pocl_bitcode_is_triple(const char *bitcode, size_t size, const char *triple) {
   std::string Triple;
   if (getModuleTriple(bitcode, size, Triple) == 0)
     return Triple.find(triple) != std::string::npos;
@@ -188,6 +345,7 @@ void cpu_setup_vector_widths(cl_device_id dev) {
     if (features["avx512f"])
       lane_width = 64;
   }
+  dev->native_vector_width_in_bits = lane_width * 8;
 
   dev->native_vector_width_char = dev->preferred_vector_width_char =
       VECWIDTH(cl_char);
@@ -236,15 +394,17 @@ static void diagHandler(LLVMDiagnosticInfoRef DI, void *diagprinter) {
   unwrap(DI)->print(*poclDiagPrinter);
 }
 
-std::string getDiagString(cl_context ctx) {
-  PoclLLVMContextData *llvm_ctx = (PoclLLVMContextData *)ctx->llvm_context_data;
-
+std::string getDiagString(void *PoclCtx) {
+  PoclLLVMContextData *llvm_ctx = (PoclLLVMContextData *)PoclCtx;
   llvm_ctx->poclDiagStream->flush();
   std::string ret(*llvm_ctx->poclDiagString);
   llvm_ctx->poclDiagString->clear();
   return ret;
 }
 
+std::string getDiagString(cl_context ctx) {
+  return getDiagString((PoclLLVMContextData *)ctx->llvm_context_data);
+}
 
 /* The LLVM API interface functions are not at the moment not thread safe,
  * Pocl needs to ensure only one thread is using this layer at the time.
@@ -260,7 +420,7 @@ std::string CurrentWgMethod;
 
 static bool LLVMInitialized = false;
 static bool LLVMOptionsInitialized = false;
-
+static bool LLVMUseGlobalContext = true;
 /* must be called with kernelCompilerLock locked */
 void InitializeLLVM() {
 
@@ -289,6 +449,11 @@ void InitializeLLVM() {
     initializeTarget(Registry);
   }
 
+  if (pocl_get_bool_option("POCL_LLVM_GLOBAL_CONTEXT", 1) == 1)
+    LLVMUseGlobalContext = true;
+  else
+    LLVMUseGlobalContext = false;
+
   // Set the options only once. TODO: fix it so that each
   // device can reset their own options. Now one cannot compile
   // with different options to different devices at one run.
@@ -378,27 +543,19 @@ one times!
 */
 
 void UnInitializeLLVM() {
-  clearKernelPasses();
-  clearTargetMachines();
   LLVMInitialized = false;
 }
 
-#define GLOBAL_LLVM_CONTEXT
-
-#ifdef GLOBAL_LLVM_CONTEXT
 static PoclLLVMContextData *GlobalLLVMContext = nullptr;
 static unsigned GlobalLLVMContextRefcount = 0;
-#endif
 
 void pocl_llvm_create_context(cl_context ctx) {
 
-#ifdef GLOBAL_LLVM_CONTEXT
-  if (GlobalLLVMContext != nullptr) {
+  if (LLVMUseGlobalContext && GlobalLLVMContext != nullptr) {
     ctx->llvm_context_data = GlobalLLVMContext;
     ++GlobalLLVMContextRefcount;
     return;
   }
-#endif
 
   PoclLLVMContextData *data = new PoclLLVMContextData;
   assert(data);
@@ -427,10 +584,10 @@ void pocl_llvm_create_context(cl_context ctx) {
                                   (void *)data->poclDiagPrinter);
   assert(ctx->llvm_context_data == nullptr);
   ctx->llvm_context_data = data;
-#ifdef GLOBAL_LLVM_CONTEXT
-  GlobalLLVMContext = data;
-  ++GlobalLLVMContextRefcount;
-#endif
+  if (LLVMUseGlobalContext) {
+    GlobalLLVMContext = data;
+    ++GlobalLLVMContextRefcount;
+  }
 
   POCL_MSG_PRINT_LLVM("Created context %" PRId64 " (%p)\n", ctx->id, ctx);
 }
@@ -439,14 +596,16 @@ void pocl_llvm_release_context(cl_context ctx) {
 
   POCL_MSG_PRINT_LLVM("releasing LLVM context\n");
 
-#ifdef GLOBAL_LLVM_CONTEXT
-  --GlobalLLVMContextRefcount;
-  if (GlobalLLVMContextRefcount > 0)
-    return;
-#endif
-
   PoclLLVMContextData *data = (PoclLLVMContextData *)ctx->llvm_context_data;
-  assert(data);
+  // this can happen if clContextCreate runs into an error
+  if (data == NULL)
+    return;
+
+  if (LLVMUseGlobalContext) {
+    --GlobalLLVMContextRefcount;
+    if (GlobalLLVMContextRefcount > 0)
+      return;
+  }
 
   if (data->number_of_IRs > 0) {
     POCL_ABORT("still have references to IRs - can't release LLVM context !\n");
@@ -470,9 +629,9 @@ void pocl_llvm_release_context(cl_context ctx) {
   delete data->Context;
   delete data;
   ctx->llvm_context_data = nullptr;
-#ifdef GLOBAL_LLVM_CONTEXT
-  GlobalLLVMContext = nullptr;
-#endif
+  if (LLVMUseGlobalContext) {
+    GlobalLLVMContext = nullptr;
+  }
 }
 
 #define POCL_METADATA_ROOT "pocl_meta"
