@@ -51,6 +51,7 @@
 #include "pocl_local_size.h"
 #include "pocl_mem_management.h"
 #include "pocl_runtime_config.h"
+#include "pocl_shared.h"
 #include "pocl_timing.h"
 #include "pocl_util.h"
 #include "utlist.h"
@@ -675,12 +676,19 @@ ERROR:
   return errcode;
 }
 
+/**
+ * @param dev Destination device
+ * @param ev_export_p Optional output parameter for the export event
+ * @param migration_size Max number of bytes to migrate (caller has to read
+ *                       content size from mem->size_buffer if applicable)
+ */
 static int
-pocl_create_migration_commands (cl_device_id dev, cl_event final_event,
-                                cl_mem mem, pocl_mem_identifier *p,
-                                const char readonly,
+pocl_create_migration_commands (cl_device_id dev, cl_event *ev_export_p,
+                                cl_event final_event, cl_mem mem,
+                                pocl_mem_identifier *p, const char readonly,
                                 cl_command_type command_type,
-                                cl_mem_migration_flags mig_flags)
+                                cl_mem_migration_flags mig_flags,
+                                uint64_t migration_size)
 {
   int errcode = CL_SUCCESS;
 
@@ -796,7 +804,8 @@ pocl_create_migration_commands (cl_device_id dev, cl_event final_event,
         {
           mem->mem_host_ptr_version = mem->latest_version;
           /* migrate content only if needed */
-          if ((mig_flags & CL_MIGRATE_MEM_OBJECT_CONTENT_UNDEFINED) == 0)
+          if ((mig_flags & CL_MIGRATE_MEM_OBJECT_CONTENT_UNDEFINED) == 0
+              || migration_size == 0)
             {
               /* Could be that destination dev has the latest version,
                * we still need to migrate to host mem */
@@ -815,9 +824,18 @@ pocl_create_migration_commands (cl_device_id dev, cl_event final_event,
   /* otherwise, we're migrating to a device memory. */
   /* check if we can migrate to the device associated with command_queue
    * without incurring the overhead of migrating their contents */
-  if (mig_flags & CL_MIGRATE_MEM_OBJECT_CONTENT_UNDEFINED)
+  if (mig_flags & CL_MIGRATE_MEM_OBJECT_CONTENT_UNDEFINED
+      || migration_size == 0)
     p->version = mem->latest_version;
 
+  can_directly_mig = highest_d2d_mig_priority > 0;
+
+  /* Set the flag so the hostptr refcount gets incremented to keep the buffer
+   * alive until it has been read for the associated content buffer's
+   * migration. */
+  if ((mem->content_buffer != NULL) && !can_directly_mig)
+    do_need_hostptr = 1;
+
   /* if we don't need to migrate, skip to end */
   if (p->version >= mem->latest_version)
     {
@@ -826,8 +844,6 @@ pocl_create_migration_commands (cl_device_id dev, cl_event final_event,
       goto FINISH_VER_SETUP;
     }
 
-  can_directly_mig = highest_d2d_mig_priority > 0;
-
   /* if mem_host_ptr is outdated AND the devices can't migrate
    * between each other, we need an export command */
   if ((mem->mem_host_ptr_version != mem->latest_version)
@@ -870,16 +886,21 @@ FINISH_VER_SETUP:
 
   if (do_need_hostptr)
     {
-      /* increase refcount the two mig commands */
+      /* increase refcount for the two mig commands and for the caller
+       * if this is a size buffer needed for content size -aware migration */
       if (do_export)
         ++mem->mem_host_ptr_refcount;
       if (do_import)
         ++mem->mem_host_ptr_refcount;
+      if (ev_export_p)
+        ++mem->mem_host_ptr_refcount;
 
       /* allocate mem_host_ptr here if needed... */
       if (mem->mem_host_ptr == NULL)
         {
           size_t align = max (mem->context->min_buffer_alignment, 16);
+          /* Always allocate mem_host_ptr for the full size of the buffer to
+           * guard against applications forgetting to check content size */
           mem->mem_host_ptr = pocl_aligned_malloc (align, mem->size);
           assert ((mem->mem_host_ptr != NULL)
                   && "Cannot allocate backing memory for mem_host_ptr!\n");
@@ -910,10 +931,17 @@ FINISH_VER_SETUP:
       cmd_export->command.migrate.mem_id
           = &mem->device_ptrs[ex_dev->global_mem_id];
       cmd_export->command.migrate.type = ENQUEUE_MIGRATE_TYPE_D2H;
+      cmd_export->command.migrate.migration_size = migration_size;
 
       pocl_command_enqueue (ex_cq, cmd_export);
 
       last_migration_event = ev_export;
+
+      if (ev_export_p)
+        {
+          POname (clRetainEvent) (ev_export);
+          *ev_export_p = ev_export;
+        }
     }
 
   /* enqueue a command for import.
@@ -944,12 +972,16 @@ FINISH_VER_SETUP:
               = &mem->device_ptrs[ex_dev->global_mem_id];
           cmd_import->command.migrate.dst_id
               = &mem->device_ptrs[dev->global_mem_id];
+          if (mem->size_buffer != NULL)
+            cmd_import->command.migrate.src_content_size_mem_id
+                = &mem->size_buffer->device_ptrs[ex_dev->global_mem_id];
         }
       else
         {
           cmd_import->command.migrate.type = ENQUEUE_MIGRATE_TYPE_H2D;
           cmd_import->command.migrate.mem_id
               = &mem->device_ptrs[dev->global_mem_id];
+          cmd_import->command.migrate.migration_size = migration_size;
         }
 
       pocl_command_enqueue (dev_cq, cmd_import);
@@ -1001,7 +1033,7 @@ pocl_create_command_full (_cl_command_node **cmd,
   int err = CL_SUCCESS;
   size_t i;
 
-  POCL_RETURN_ERROR_ON ((dev->available == CL_FALSE), CL_INVALID_DEVICE,
+  POCL_RETURN_ERROR_ON ((*dev->available == CL_FALSE), CL_INVALID_DEVICE,
                         "device is not available\n");
 
   if (num_buffers >= 1)
@@ -1033,12 +1065,81 @@ pocl_create_command_full (_cl_command_node **cmd,
   final_event->pocl_refcount += num_buffers;
   POCL_UNLOCK_OBJ (final_event);
 
+  cl_event *size_events = NULL;
+  /* Temporary copy of the buffer list just for keeping track of which buffers
+   * were migrated in the size buffer migration phase */
+  int *already_migrated = NULL;
+  if (num_buffers > 0)
+    {
+      size_events = alloca (sizeof (cl_event) * num_buffers);
+      memset (size_events, 0, sizeof (cl_event) * num_buffers);
+
+      already_migrated = alloca (sizeof (int) * num_buffers);
+      memset (already_migrated, 0, sizeof (int) * num_buffers);
+    }
+
+  /* Always migrate content size buffers first, if they exist */
   for (i = 0; i < num_buffers; ++i)
     {
+      if (buffers[i]->size_buffer != NULL)
+        {
+          /* Bump "last event" refcount for content size buffers that weren't
+           * explicitly given as dependencies */
+          int explicit = 0;
+          for (int j = 0; j < num_buffers; ++j)
+            {
+              if (buffers[j] == buffers[i]->size_buffer)
+                {
+                  already_migrated[j] = 1;
+                  explicit = 1;
+                  break;
+                }
+            }
+          if (!explicit)
+            {
+              POname (clRetainEvent) (final_event);
+            }
+
+          pocl_create_migration_commands (
+              dev, &size_events[i], final_event, buffers[i]->size_buffer,
+              &(buffers[i]->size_buffer)->device_ptrs[dev->global_mem_id],
+              readonly_flags[i], command_type, mig_flags,
+              buffers[i]->size_buffer->size);
+        }
+    }
+
+  for (i = 0; i < num_buffers; ++i)
+    {
+      uint64_t migration_size = buffers[i]->size;
+
+      /* If both a content buffer and its associated size buffer are explicitly
+       * listed in buffers, the size buffer was already migrated above. */
+      if (already_migrated[i])
+        continue;
+
+      if (buffers[i]->size_buffer != NULL)
+        {
+          /* BLOCK until size buffer has been imported to host mem! No event
+           * exists if host import is not needed. */
+          if (size_events[i] != NULL)
+            {
+              cl_device_id d = size_events[i]->queue->device;
+              POname (clWaitForEvents) (1, &size_events[i]);
+              if (buffers[i]->size_buffer->mem_host_ptr != NULL)
+                {
+                  migration_size
+                      = *(uint64_t *)buffers[i]->size_buffer->mem_host_ptr;
+                }
+              pocl_release_mem_host_ptr (buffers[i]->size_buffer);
+              POname (clReleaseEvent) (size_events[i]);
+              size_events[i] = NULL;
+            }
+        }
+
       pocl_create_migration_commands (
-          dev, final_event, buffers[i],
+          dev, NULL, final_event, buffers[i],
           &buffers[i]->device_ptrs[dev->global_mem_id], readonly_flags[i],
-          command_type, mig_flags);
+          command_type, mig_flags, migration_size);
     }
 
   return err;
@@ -1852,16 +1953,6 @@ pocl_status_to_str (int status)
   return status_to_str[status];
 }
 
-void
-pocl_abort_on_pthread_error (int status, unsigned line, const char *func)
-{
-  if (status != 0)
-    {
-      POCL_MSG_PRINT2 (ERROR, func, line, "Error from pthread call:\n");
-      POCL_ABORT ("PTHREAD ERROR in %s():%u: %s (%d)\n", func, line, strerror (status), status);
-    }
-}
-
 /* Convert a command type to its representation string
  */
 const char *
@@ -2137,6 +2228,7 @@ pocl_update_event_running (cl_event event)
   POCL_UNLOCK_OBJ (event);
 }
 
+/* Note: this must be kept in sync with pocl_copy_event_node */
 static void pocl_free_event_node (cl_event event)
 {
   _cl_command_node *node = event->command;
@@ -2148,11 +2240,11 @@ static void pocl_free_event_node (cl_event event)
       break;
 
     case CL_COMMAND_FILL_BUFFER:
-      pocl_aligned_free (node->command.memfill.pattern);
+      POCL_MEM_FREE (node->command.memfill.pattern);
       break;
 
     case CL_COMMAND_SVM_MEMFILL:
-      pocl_aligned_free (node->command.svm_fill.pattern);
+      POCL_MEM_FREE (node->command.svm_fill.pattern);
       break;
 
     case CL_COMMAND_NATIVE_KERNEL:
@@ -2176,6 +2268,64 @@ static void pocl_free_event_node (cl_event event)
   event->command = NULL;
 }
 
+int
+pocl_copy_event_node (_cl_command_node *dst_node, _cl_command_node *src_node)
+{
+  memcpy (&dst_node->command, &src_node->command, sizeof (_cl_command_t));
+  // Copy variables that are freed when the command finishes
+  switch (src_node->type)
+    {
+    case CL_COMMAND_NDRANGE_KERNEL:
+    case CL_COMMAND_TASK:
+      POname (clRetainKernel) (src_node->command.run.kernel);
+      /* note: this must use the arguments stored in the src_node,
+       * NOT the ones in kernel->dyn_arguments; these might differ,
+       * because the user could clSetKernelArg() right after
+       * clCommandNDRangeKernelKHR(). */
+      int errcode = pocl_kernel_copy_args (src_node->command.run.kernel,
+                                           src_node->command.run.arguments,
+                                           &dst_node->command.run);
+      if (errcode != CL_SUCCESS)
+        return CL_OUT_OF_HOST_MEMORY;
+      break;
+
+    case CL_COMMAND_FILL_BUFFER:
+      dst_node->command.memfill.pattern
+          = pocl_aligned_malloc (src_node->command.memfill.pattern_size,
+                                 src_node->command.memfill.pattern_size);
+      if (dst_node->command.memfill.pattern == NULL)
+        return CL_OUT_OF_HOST_MEMORY;
+      memcpy (dst_node->command.memfill.pattern,
+              src_node->command.memfill.pattern,
+              src_node->command.memfill.pattern_size);
+      break;
+
+    case CL_COMMAND_SVM_MEMFILL:
+      dst_node->command.svm_fill.pattern
+          = pocl_aligned_malloc (src_node->command.svm_fill.pattern_size,
+                                 src_node->command.svm_fill.pattern_size);
+      if (dst_node->command.svm_fill.pattern == NULL)
+        return CL_OUT_OF_HOST_MEMORY;
+      memcpy (dst_node->command.svm_fill.pattern,
+              src_node->command.svm_fill.pattern,
+              src_node->command.svm_fill.pattern_size);
+      break;
+
+#if 0
+    /* these cases are currently not handled in pocl_copy_event_node,
+     * because there is no command buffer equivalent of these nodes. */
+    case CL_COMMAND_NATIVE_KERNEL:
+    case CL_COMMAND_UNMAP_MEM_OBJECT:
+    case CL_COMMAND_SVM_MIGRATE_MEM:
+    case CL_COMMAND_SVM_FREE:
+#endif
+
+    default:
+      break;
+    }
+  return CL_SUCCESS;
+}
+
 static void pocl_free_event_memobjs (cl_event event)
 {
   size_t i;
@@ -2237,6 +2387,42 @@ pocl_update_event_finished (cl_int status, const char *func, unsigned line,
   POCL_UNLOCK_OBJ (event);
   ops->broadcast (event);
 
+  /* With remote being asynchronous it is possible that an event is signaled as
+   * complete before some of its dependencies. Therefore this event has to be
+   * removed from the notify lists of any remaining events in the wait list.
+   *
+   * Mind the acrobatics of trying to avoid races with pocl_broadcast and
+   * pocl_create_event_sync. */
+  event_node *tmp;
+  POCL_LOCK_OBJ (event);
+  while ((tmp = event->wait_list))
+    {
+      cl_event notifier = tmp->event;
+      POCL_UNLOCK_OBJ (event);
+      pocl_lock_events_inorder (notifier, event);
+      if (tmp != event->wait_list)
+        {
+          pocl_unlock_events_inorder (notifier, event);
+          POCL_LOCK_OBJ (event);
+          continue;
+        }
+      event_node *tmp2;
+      LL_FOREACH (notifier->notify_list, tmp2)
+      {
+        if (tmp2->event == event)
+          {
+            LL_DELETE (notifier->notify_list, tmp2);
+            pocl_mem_manager_free_event_node (tmp2);
+            break;
+          }
+      }
+      LL_DELETE (event->wait_list, tmp);
+      pocl_unlock_events_inorder (notifier, event);
+      pocl_mem_manager_free_event_node (tmp);
+      POCL_LOCK_OBJ (event);
+    }
+  POCL_UNLOCK_OBJ (event);
+
 #ifdef POCL_DEBUG_MESSAGES
   if (msg != NULL)
     {
@@ -2269,6 +2455,14 @@ pocl_update_event_failed (cl_event event)
   POCL_LOCK_OBJ (event);
 }
 
+void
+pocl_update_event_device_lost (cl_event event)
+{
+  POCL_UNLOCK_OBJ (event);
+  pocl_update_event_finished (CL_DEVICE_NOT_AVAILABLE, NULL, 0, event, NULL);
+  POCL_LOCK_OBJ (event);
+}
+
 void
 pocl_update_event_complete (const char *func, unsigned line,
                             cl_event event, const char *msg)
