@@ -34,14 +34,25 @@
 
 #define OCL_C_AS
 
+#if 0
 /* The OpenCL printf routine.
  *
  * For debugging
  * Use as: DEBUG_PRINTF((fmt, args...)) -- note double parentheses!
-
-int printf(const char *format, ...);
-#define DEBUG_PRINTF(args) (printf args)
 */
+extern int dprintf(int fd, const char *format, ...);
+extern int vdprintf(int fd, const char *format, va_list ap);
+int debugprintf(const char *fmt, ...)
+{
+  int ret;
+  va_list vargs;
+  va_start(vargs, fmt);
+  ret = vdprintf(2, fmt, vargs);
+  va_end(vargs);
+  return ret;
+}
+#define DEBUG_PRINTF(args) (debugprintf args)
+#endif
 
 #define DEBUG_PRINTF(args) ((void)0)
 
@@ -175,9 +186,11 @@ __pocl_printf_format_full (const PRINTF_FMT_STR_AS char *restrict format,
 {
   DEBUG_PRINTF (("[printf:format=%s]\n", format));
   char bf[BUFSIZE];
+  for (unsigned i = 0; i < BUFSIZE; ++i)
+    bf[i] = 0;
   p->bf = bf;
-  char ch;
-  unsigned errcode;
+  char ch = 0;
+  unsigned errcode = 0;
 
   while ((ch = *format++))
     {
@@ -208,6 +221,7 @@ __pocl_printf_format_full (const PRINTF_FMT_STR_AS char *restrict format,
               flags.zero = 0;
               flags.uc = 0;
               flags.always_sign = 0;
+              flags.nonzeroparam = 0;
               for (;;)
                 {
                   switch (ch)
