@@ -191,7 +191,9 @@ void dumpCFG(
 
   s << "}" << std::endl;
   s.close();
+#if 0
   std::cout << "### dumped CFG to " << fname << std::endl;
+#endif
 }
 
 bool chopBBs(llvm::Function &F, llvm::Pass &) {
@@ -220,4 +222,62 @@ bool chopBBs(llvm::Function &F, llvm::Pass &) {
   } while (fchanged);
   return fchanged;
 }
-};
+
+void PoCLCFGPrinter::dumpModule(llvm::Module &M) {
+  for (llvm::Function &F : M) {
+    std::string Name;
+    if (F.hasName())
+      Name += F.getName();
+    else
+      Name += "anonymous_func";
+    Name = Prefix + Name + ".dot";
+    // TODO somehow supply regions/highlights
+    dumpCFG(F, Name, nullptr, nullptr);
+  }
+}
+
+#if LLVM_MAJOR < MIN_LLVM_NEW_PASSMANAGER
+char PoCLCFGPrinter::ID = 0;
+
+bool PoCLCFGPrinter::runOnModule(Module &M) {
+  dumpModule(M);
+  return false;
+}
+
+void PoCLCFGPrinter::getAnalysisUsage(llvm::AnalysisUsage &AU) const {
+  AU.setPreservesAll();
+}
+
+static llvm::RegisterPass<PoCLCFGPrinter>
+    X("print-pocl-cfg", "Print PoCL-style CFG of the Module");
+
+#else
+
+llvm::PreservedAnalyses PoCLCFGPrinter::run(llvm::Module &M,
+                                            llvm::ModuleAnalysisManager &AM) {
+  dumpModule(M);
+  return PreservedAnalyses::all();
+}
+
+void PoCLCFGPrinter::registerWithPB(llvm::PassBuilder &PB) {
+  PB.registerPipelineParsingCallback(
+      [](::llvm::StringRef Name, ::llvm::ModulePassManager &MPM,
+         llvm::ArrayRef<::llvm::PassBuilder::PipelineElement>) {
+        if (Name == "print<pocl-cfg>") {
+          MPM.addPass(PoCLCFGPrinter(llvm::errs()));
+          return true;
+        }
+        // the string X in "print<pocl-cfg;X>" will be passed to constructor;
+        // this can be used to run multiple times and dump to different files
+        if (Name.consume_front("print<pocl-cfg;") && Name.consume_back(">")) {
+          MPM.addPass(PoCLCFGPrinter(llvm::errs(), Name));
+          return true;
+        }
+
+        return false;
+      });
+}
+
+#endif
+
+} // namespace pocl
