@@ -106,13 +106,15 @@ poclu_get_any_device2 (cl_context *context, cl_device_id *device,
 
 cl_int
 poclu_get_multiple_devices (cl_platform_id *platform, cl_context *context,
-                            cl_char include_custom_dev,
-                            cl_uint *num_devices, cl_device_id **devices,
-                            cl_command_queue **queues)
+                            cl_char include_custom_dev, cl_uint *num_devices,
+                            cl_device_id **devices, cl_command_queue **queues,
+                            int ooo_queues)
 {
   cl_int err;
   cl_uint num_dev_all = 0;
   cl_uint num_dev_custom = 0;
+  cl_device_id *devs = NULL;
+  cl_command_queue *ques = NULL;
   size_t i;
 
   if (context == NULL || devices == NULL || queues == NULL || platform == NULL)
@@ -120,7 +122,7 @@ poclu_get_multiple_devices (cl_platform_id *platform, cl_context *context,
 
   err = clGetPlatformIDs (1, platform, NULL);
   if (err != CL_SUCCESS)
-    return err;
+    goto ERROR;
 
   err = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_ALL, 0, NULL, &num_dev_all);
 
@@ -128,7 +130,7 @@ poclu_get_multiple_devices (cl_platform_id *platform, cl_context *context,
   if (include_custom_dev != 1)
     {
       if (err != CL_SUCCESS)
-        return err;
+        goto ERROR;
     }
   else
     {
@@ -137,7 +139,8 @@ poclu_get_multiple_devices (cl_platform_id *platform, cl_context *context,
 
       if (err != CL_SUCCESS && err_custom != CL_SUCCESS)
         {
-          return err_custom;
+          err = err_custom;
+          goto ERROR;
         }
       if (err != CL_SUCCESS)
         {
@@ -152,17 +155,15 @@ poclu_get_multiple_devices (cl_platform_id *platform, cl_context *context,
 
   *num_devices = num_dev_all + num_dev_custom;
 
-  cl_device_id *devs
-      = (cl_device_id *) calloc (*num_devices, sizeof (cl_device_id));
-  cl_command_queue *ques
-      = (cl_command_queue *) calloc (*num_devices, sizeof (cl_command_queue));
+  devs = (cl_device_id *)calloc (*num_devices, sizeof (cl_device_id));
+  ques = (cl_command_queue *)calloc (*num_devices, sizeof (cl_command_queue));
 
   if (num_dev_all != 0)
     {
       err = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_ALL, num_dev_all, devs,
                             NULL);
       if (err != CL_SUCCESS)
-        return err;
+        goto ERROR;
     }
 
   if (num_dev_custom != 0)
@@ -171,24 +172,31 @@ poclu_get_multiple_devices (cl_platform_id *platform, cl_context *context,
       err = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_CUSTOM, num_dev_custom, &devs[num_dev_all],
                             NULL);
       if (err != CL_SUCCESS)
-        return err;
+        goto ERROR;
     }
 
   *context = clCreateContext (NULL, *num_devices, devs, NULL, NULL, &err);
   if (err != CL_SUCCESS)
-    return err;
+    goto ERROR;
 
+  cl_command_queue_properties props = CL_QUEUE_PROFILING_ENABLE;
+  if (ooo_queues)
+    props |= CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE;
   for (i = 0; i < *num_devices; ++i)
     {
-      ques[i] = clCreateCommandQueue (*context, devs[i],
-                                      CL_QUEUE_PROFILING_ENABLE, &err);
+      ques[i] = clCreateCommandQueue (*context, devs[i], props, &err);
       if (err != CL_SUCCESS)
-        return err;
+        goto ERROR;
     }
 
   *devices = devs;
   *queues = ques;
   return CL_SUCCESS;
+
+ERROR:
+  free (devs);
+  free (ques);
+  return err;
 }
 
 char *
@@ -552,9 +560,11 @@ poclu_load_program_multidev (cl_context context, cl_device_id *devices,
                              NULL);
       CHECK_OPENCL_ERROR_IN ("clGetDeviceInfo extensions");
 
-      if (strstr (extensions, "cl_khr_spir") == NULL)
+      if ((spir && strstr (extensions, "cl_khr_spir") == NULL)
+          || (spirv && strstr (extensions, "cl_khr_il_program") == NULL))
         {
-          printf ("SPIR not supported, cannot run the test\n");
+          printf ("SPIR%s not supported, cannot run the test\n",
+                  spirv ? "-V" : "");
           return -1;
         }
 
