@@ -16,7 +16,6 @@ int main(int argc, char **argv)
   size_t srcdir_length, name_length, filename_size;
   char *filename = NULL;
   char *source = NULL;
-  cl_device_id devices[1];
   cl_context context = NULL;
   cl_command_queue queue = NULL;
   cl_program program = NULL;
@@ -71,11 +70,21 @@ int main(int argc, char **argv)
       goto error;
     }
 
-  result = clGetContextInfo(context, CL_CONTEXT_DEVICES,
-                            sizeof(cl_device_id), devices, NULL);
-  if (result != CL_SUCCESS) 
+  size_t device_id_size = 0;
+  result = clGetContextInfo (context, CL_CONTEXT_DEVICES, 0, NULL,
+                             &device_id_size);
+  if (result != CL_SUCCESS)
     {
-      puts("clGetContextInfo call failed\n");
+      puts ("clGetContextInfo call failed while fetching size\n");
+      goto error;
+    }
+  cl_device_id *devices = malloc (device_id_size);
+  TEST_ASSERT (devices != NULL && "out of host memory\n");
+  result = clGetContextInfo (context, CL_CONTEXT_DEVICES, device_id_size,
+                             devices, NULL);
+  if (result != CL_SUCCESS)
+    {
+      puts ("clGetContextInfo call failed\n");
       goto error;
     }
 
@@ -176,6 +185,7 @@ error:
     {
       free(imageData);
     }
+  free (devices);
 
   CHECK_CL_ERROR (clUnloadCompiler ());
 
