@@ -21,11 +21,13 @@
    THE SOFTWARE.
 */
 
+#include "pocl_opencl.h"
+
 // Enable OpenCL C++ exceptions
 #define CL_HPP_ENABLE_EXCEPTIONS
 #define CL_HPP_MINIMUM_OPENCL_VERSION 120
 #define CL_HPP_TARGET_OPENCL_VERSION 120
-#include <CL/cl2.hpp>
+#include <CL/opencl.hpp>
 
 #include <cstdio>
 #include <cstdlib>
@@ -139,6 +141,7 @@ main(void)
         platformList[0].unloadCompiler();
 
         // If the kernel compiler succeeds, we are happy for now.
+        std::cout << "OK" << std::endl;
         return EXIT_SUCCESS;
     } 
     catch (cl::Error &err) {
@@ -149,9 +152,7 @@ main(void)
              << err.err()
              << ")"
              << std::endl;
-
-         return EXIT_FAILURE;
     }
 
-    return EXIT_SUCCESS;
+    return EXIT_FAILURE;
 }
