@@ -21,11 +21,13 @@
    THE SOFTWARE.
 */
 
+#include "pocl_opencl.h"
+
 // Enable OpenCL C++ exceptions
 #define CL_HPP_ENABLE_EXCEPTIONS
 #define CL_HPP_MINIMUM_OPENCL_VERSION 120
 #define CL_HPP_TARGET_OPENCL_VERSION 120
-#include <CL/cl2.hpp>
+#include <CL/opencl.hpp>
 
 #include <cstdio>
 #include <cstdlib>
@@ -165,10 +167,10 @@ main(void)
         queue.finish();
         platformList[0].unloadCompiler();
 
-        if (ok)
+        if (ok) {
+            std::cout << "OK" << std::endl;
             return EXIT_SUCCESS;
-        else
-            return EXIT_FAILURE;
+        }
     }
     catch (cl::Error &err) {
          std::cerr
@@ -178,9 +180,7 @@ main(void)
              << err.err()
              << ")"
              << std::endl;
-
-         return EXIT_FAILURE;
     }
 
-    return EXIT_SUCCESS;
+    return EXIT_FAILURE;
 }
