@@ -163,9 +163,10 @@ main(void){
   
   binary_sizes = (size_t*)malloc(num_binaries * sizeof(size_t));
   binaries = (const unsigned char**)calloc(num_binaries, sizeof(unsigned char*));
-  
-  err = clGetProgramInfo(program, CL_PROGRAM_BINARY_SIZES, 1*sizeof(size_t), 
-			 binary_sizes , &num_bytes_copied);
+
+  err = clGetProgramInfo (program, CL_PROGRAM_BINARY_SIZES,
+                          num_binaries * sizeof (size_t), binary_sizes,
+                          &num_bytes_copied);
   CHECK_OPENCL_ERROR_IN("clGetProgramInfo");
   
   binary_sizes[1] = binary_sizes[0];
@@ -174,9 +175,10 @@ main(void){
 					      sizeof(const unsigned char));
   binaries[1] = (const unsigned char*) malloc(binary_sizes[1] *
 					      sizeof(const unsigned char));
-  
-  err = clGetProgramInfo(program, CL_PROGRAM_BINARIES, 1 * sizeof(char*), 
-			 binaries, &num_bytes_copied);
+
+  err = clGetProgramInfo (program, CL_PROGRAM_BINARIES,
+                          num_binaries * sizeof (char *), binaries,
+                          &num_bytes_copied);
   CHECK_OPENCL_ERROR_IN("clGetProgramInfo");
   
   memcpy((void*)binaries[1], (void*)binaries[0], binary_sizes[0]);      
