@@ -21,11 +21,11 @@
    THE SOFTWARE.
  */
 
+#include "pocl_opencl.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <CL/cl.h>
-#include "poclu.h"
 
 int main(int argc, char **argv)
 {
@@ -65,8 +65,8 @@ int main(int argc, char **argv)
   CHECK_CL_ERROR (clReleaseContext (ctx));
   CHECK_CL_ERROR (clUnloadCompiler ());
 
+  printf ("OK\n");
   return EXIT_SUCCESS;
-
 }
 
 
