@@ -20,14 +20,14 @@
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.
 */
-#include <CL/cl.h>
+
+#include "pocl_opencl.h"
+
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 
-#include "poclu.h"
-
 #define ARRAY_SIZE(arr) (sizeof(arr)/sizeof(*arr))
 
 int main()
@@ -83,6 +83,6 @@ int main()
 
   CHECK_CL_ERROR (clUnloadCompiler ());
 
+  printf ("OK\n");
   return EXIT_SUCCESS;
-
 }
