@@ -94,13 +94,6 @@ main(void)
         cl::Device device1 = devices.at(0);
         cl::Device device2 = devices.at(1);
 
-        // we need tta and pthread.
-        assert ((device1.getInfo<CL_DEVICE_NAME>().find("pthread") != std::string::npos &&
-                 device2.getInfo<CL_DEVICE_NAME>().find("tta") != std::string::npos)
-                ||
-                (device2.getInfo<CL_DEVICE_NAME>().find("pthread") != std::string::npos &&
-                 device1.getInfo<CL_DEVICE_NAME>().find("tta") != std::string::npos));
-
         // Create and program from source
         cl::Program::Sources sources({kernelSourceCode});
         cl::Program program(context, sources);
